#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import absolute_import, division, print_function

__metaclass__ = type

GENERATOR_VERSION = "1.0"


DOCUMENTATION = """
module: iosxr_l2_interfaces
short_description: L2 interfaces resource module
description: This module manages the Layer-2 interface attributes on Cisco IOS-XR
  devices.
version_added: 1.0.0
author:
- Sumit Jaiswal (@justjais)
- Rohit Thakur (@rohitthakur2590)
notes:
- Tested against Cisco IOS-XRv Version 6.1.3 on VIRL.
- This module works with connection C(network_cli). See L(the IOS-XR Platform Options,../network/user_guide/platform_iosxr.html).
options:
  config:
    description: A dictionary of Layer-2 interface options
    type: list
    elements: dict
    suboptions:
      name:
        description:
        - Full name of the interface/sub-interface excluding any logical unit number,
          e.g. GigabitEthernet0/0/0/1 or GigabitEthernet0/0/0/1.100.
        type: str
        required: true
      native_vlan:
        description:
        - Configure a native VLAN ID for the trunk
        type: int
      l2transport:
        description:
        - Switchport mode access command to configure the interface as a layer 2 access
        type: bool
      l2protocol:
        description:
        - Configures Layer 2 protocol tunneling and protocol data unit (PDU) filtering
          on an interface.
        type: list
        elements: dict
        suboptions:
          cdp:
            description:
            - Cisco Discovery Protocol (CDP) tunneling and data unit parameters.
            choices:
            - drop
            - forward
            - tunnel
            type: str
          pvst:
            description:
            - Configures the per-VLAN Spanning Tree Protocol (PVST) tunneling and
              data unit parameters.
            choices:
            - drop
            - forward
            - tunnel
            type: str
          stp:
            description:
            - Spanning Tree Protocol (STP) tunneling and data unit parameters.
            choices:
            - drop
            - forward
            - tunnel
            type: str
          vtp:
            description:
            - VLAN Trunk Protocol (VTP) tunneling and data unit parameters.
            choices:
            - drop
            - forward
            - tunnel
            type: str
      q_vlan:
        description:
        - 802.1Q VLAN configuration. Note that it can accept either 2 VLAN IDs when
          configuring Q-in-Q VLAN, or it will accept 1 VLAN ID and 'any' as input
          list when configuring Q-in-any vlan as input. Note, that this option is
          valid only with respect to Sub-Interface and is not valid when configuring
          for Interface.
        type: list
        elements: int
      propagate:
        description:
        - Propagate Layer 2 transport events. Note that it will work only when the
          I(l2tranport) option is set to TRUE
        type: bool
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the IOS-XR device
      by executing the command B(show running-config interface).
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - rendered
    - gathered
    - parsed
    default: merged
    description:
    - The state of the configuration after module completion
    type: str
"""

EXAMPLES = """
# Using merged
#
# Before state:
# -------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/3
#  description Ansible Network
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  duplex half
#  shutdown
# !
# interface GigabitEthernet0/0/0/4
#  description Test description
# !

- name: Merge provided configuration with device configuration
  cisco.iosxr.iosxr_l2_interfaces:
    config:
    - name: GigabitEthernet0/0/0/3
      native_vlan: 20
    - name: GigabitEthernet0/0/0/4
      native_vlan: 40
      l2transport: true
      l2protocol:
      - stp: tunnel
    - name: GigabitEthernet0/0/0/3.900
      l2transport: true
      q_vlan:
      - 20
      - 40
    state: merged

# After state:
# ------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/3
#  description Ansible Network
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  duplex half
#  shutdown
#  dot1q native vlan 20
# !
# interface GigabitEthernet0/0/0/4
# description Test description
#  dot1q native vlan 10
#  l2transport
#   l2protocol stp tunnel
#  !
# !
# interface GigabitEthernet0/0/0/3.900 l2transport
#  dot1q vlan 20 40
# !

# Using replaced
#
# Before state:
# -------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/3
#  description Ansible Network
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  duplex half
#  shutdown
#  dot1q native vlan 20
# !
# interface GigabitEthernet0/0/0/4
# description Test description
#  dot1q native vlan 10
#  l2transport
#   l2protocol stp tunnel
#  !
# !
# interface GigabitEthernet0/0/0/3.900 l2transport
#  dot1q vlan 20 40
# !

- name: Replaces device configuration of listed interfaces with provided configuration
  cisco.iosxr.iosxr_l2_interfaces:
    config:
    - name: GigabitEthernet0/0/0/4
      native_vlan: 40
      l2transport: true
      l2protocol:
      - stp: forward
    - name: GigabitEthernet0/0/0/3.900
      q_vlan:
      - 20
      - any
    state: replaced

# After state:
# -------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/3
#  description Ansible Network
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  duplex half
#  shutdown
#  dot1q native vlan 20
# !
# interface GigabitEthernet0/0/0/4
# description Test description
#  dot1q native vlan 40
#  l2transport
#   l2protocol stp forward
#  !
# !
# interface GigabitEthernet0/0/0/3.900 l2transport
#  dot1q vlan 20 any
# !

# Using overridden
#
# Before state:
# -------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/3
#  description Ansible Network
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  duplex half
#  shutdown
#  dot1q native vlan 20
# !
# interface GigabitEthernet0/0/0/4
# description Test description
#  dot1q native vlan 10
#  l2transport
#   l2protocol stp tunnel
#  !
# !
# interface GigabitEthernet0/0/0/3.900 l2transport
#  dot1q vlan 20 40
# !

- name: Override device configuration of all interfaces with provided configuration
  cisco.iosxr.iosxr_l2_interfaces:
    config:
    - name: GigabitEthernet0/0/0/4
      native_vlan: 40
      l2transport: true
      l2protocol:
      - stp: forward
    - name: GigabitEthernet0/0/0/3.900
      q_vlan:
      - 20
      - any
    state: overridden

# After state:
# -------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/3
#  description Ansible Network
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  duplex half
#  shutdown
# !
# interface GigabitEthernet0/0/0/4
# description Test description
#  dot1q native vlan 40
#  l2transport
#   l2protocol stp forward
#  !
# !
# interface GigabitEthernet0/0/0/3.900
#  dot1q vlan 20 any
# !

# Using deleted
#
# Before state:
# -------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/3
#  description Ansible Network
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  duplex half
#  shutdown
#  dot1q native vlan 20
# !
# interface GigabitEthernet0/0/0/4
#  description Test description
#  dot1q native vlan 10
#  l2transport
#   l2protocol stp tunnel
#  !
# !
#

- name: "Delete L2 attributes of given interfaces (Note: This won't delete the interface itself)"
  cisco.iosxr.iosxr_l2_interfaces:
    config:
    - name: GigabitEthernet0/0/0/4
    state: deleted

# After state:
# ------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/3
#  description Ansible Network
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  duplex half
#  shutdown
#  dot1q native vlan 20
# !
# interface GigabitEthernet0/0/0/4
#  description Test description
# !

# Using Deleted without any config passed
# "(NOTE: This will delete all of configured resource module attributes from each configured interface)"
#
# Before state:
# -------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/3
#  description Ansible Network
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  duplex half
#  shutdown
#  dot1q native vlan 20
# !
# interface GigabitEthernet0/0/0/4
#  description Test description
#  dot1q native vlan 10
#  l2transport
#   l2protocol stp tunnel
#  !
# !

- name: "Delete L2 attributes of all interfaces (Note: This won't delete the interface itself)"
  cisco.iosxr.iosxr_l2_interfaces:
    state: deleted

# After state:
# ------------
#
# viosxr#show running-config interface
# interface GigabitEthernet0/0/0/3
#  description Ansible Network
#  vrf custB
#  ipv4 address 10.10.0.2 255.255.255.0
#  duplex half
#  shutdown
# !
# interface GigabitEthernet0/0/0/4
#  description Test description
# !


# Using parsed
# parsed.cfg
# ------------
#
# interface Loopback888
#  description test for ansible
#  shutdown
# !
# interface MgmtEth0/0/CPU0/0
#  ipv4 address 10.8.38.70 255.255.255.0
# !
# interface GigabitEthernet0/0/0/0
#  description Configured and Merged by Ansible-Network
#  mtu 110
#  ipv4 address 172.31.1.1 255.255.255.0
#  duplex half
# !
# interface GigabitEthernet0/0/0/1
#  dot1q native vlan 10
#  l2transport
#   l2protocol cdp forward
#   l2protocol pvst tunnel
#   propagate remote-status
#  !
# !
# interface GigabitEthernet0/0/0/3
#  shutdown
# !
# interface GigabitEthernet0/0/0/3.900
#  encapsulation dot1q 20 second-dot1q 40
# !
# interface GigabitEthernet0/0/0/4
#  shutdown
#  dot1q native vlan 40
# !
- name: Convert L2 interfaces config to argspec without connecting to the appliance
  cisco.iosxr.iosxr_l2_interfaces:
    running_config: "{{ lookup('file', './parsed.cfg') }}"
    state: parsed
# Task Output (redacted)
# -----------------------
# "parsed": [
#         {
#             "name": "GigabitEthernet0/0/0/0"
#         },
#         {
#             "l2protocol": [
#                 {
#                     "cdp": "forward"
#                 },
#                 {
#                     "pvst": "tunnel"
#                 }
#             ],
#             "l2transport": true,
#             "name": "GigabitEthernet0/0/0/1",
#             "native_vlan": 10,
#             "propagate": true
#         },
#         {
#             "name": "GigabitEthernet0/0/0/3"
#         },
#         {
#             "name": "GigabitEthernet0/0/0/3.900",
#             "q_vlan": [
#                 20,
#                 40
#             ]
#         },
#         {
#             "name": "GigabitEthernet0/0/0/4",
#             "native_vlan": 40
#         }
#     ]


# Using rendered
- name: Render platform specific commands from task input using rendered state
  cisco.iosxr.iosxr_l2_interfaces:
    config:

    - name: GigabitEthernet0/0/0/1
      native_vlan: 10
      l2transport: true
      l2protocol:

      - pvst: tunnel

      - cdp: forward
      propagate: true

    - name: GigabitEthernet0/0/0/3.900
      q_vlan:
      - 20
      - 40

    - name: GigabitEthernet0/0/0/4
      native_vlan: 40
    state: rendered
# Task Output (redacted)
# -----------------------
# "rendered": [
#         "interface GigabitEthernet0/0/0/1",
#         "dot1q native vlan 10",
#         "l2transport l2protocol pvst tunnel",
#         "l2transport l2protocol cdp forward",
#         "l2transport propagate remote-status",
#         "interface GigabitEthernet0/0/0/3.900",
#         "dot1q vlan 20 40",
#         "interface GigabitEthernet0/0/0/4",
#         "dot1q native vlan 40"
#     ]


# Using gathered
# Before state:
# ------------
#
# RP/0/0/CPU0:an-iosxr-02#show running-config  interface
# interface Loopback888
#  description test for ansible
#  shutdown
# !
# interface MgmtEth0/0/CPU0/0
#  ipv4 address 10.8.38.70 255.255.255.0
# !
# interface GigabitEthernet0/0/0/0
#  description Configured and Merged by Ansible-Network
#  mtu 110
#  ipv4 address 172.31.1.1 255.255.255.0
#  duplex half
# !
# interface GigabitEthernet0/0/0/1
#  dot1q native vlan 10
#  l2transport
#   l2protocol cdp forward
#   l2protocol pvst tunnel
#   propagate remote-status
#  !
# !
# interface GigabitEthernet0/0/0/3
#  shutdown
# !
# interface GigabitEthernet0/0/0/3.900
#  encapsulation dot1q 20 second-dot1q 40
# !
# interface GigabitEthernet0/0/0/4
#  shutdown
#  dot1q native vlan 40
# !
- name: Gather IOSXR l2 interfaces as in given arguments
  cisco.iosxr.iosxr_l2_interfaces:
    config:
    state: gathered
# Task Output (redacted)
# -----------------------
#
# "gathered": [
#         {
#             "name": "GigabitEthernet0/0/0/0"
#         },
#         {
#             "l2protocol": [
#                 {
#                     "cdp": "forward"
#                 },
#                 {
#                     "pvst": "tunnel"
#                 }
#             ],
#             "l2transport": true,
#             "name": "GigabitEthernet0/0/0/1",
#             "native_vlan": 10,
#             "propagate": true
#         },
#         {
#             "name": "GigabitEthernet0/0/0/3"
#         },
#         {
#             "name": "GigabitEthernet0/0/0/3.900",
#             "q_vlan": [
#                 20,
#                 40
#             ]
#         },
#         {
#             "name": "GigabitEthernet0/0/0/4",
#             "native_vlan": 40
#         }
#     ]
# After state:
# ------------
#
# RP/0/0/CPU0:an-iosxr-02#show running-config  interface
# interface Loopback888
#  description test for ansible
#  shutdown
# !
# interface MgmtEth0/0/CPU0/0
#  ipv4 address 10.8.38.70 255.255.255.0
# !
# interface GigabitEthernet0/0/0/0
#  description Configured and Merged by Ansible-Network
#  mtu 110
#  ipv4 address 172.31.1.1 255.255.255.0
#  duplex half
# !
# interface GigabitEthernet0/0/0/1
#  dot1q native vlan 10
#  l2transport
#   l2protocol cdp forward
#   l2protocol pvst tunnel
#   propagate remote-status
#  !
# !
# interface GigabitEthernet0/0/0/3
#  shutdown
# !
# interface GigabitEthernet0/0/0/3.900
#  encapsulation dot1q 20 second-dot1q 40
# !
# interface GigabitEthernet0/0/0/4
#  shutdown
#  dot1q native vlan 40
# !



"""

RETURN = """
before:
  description: The configuration as structured data prior to module invocation.
  returned: always
  type: list
  sample: The configuration returned will always be in the same format of the parameters above.
after:
  description: The configuration as structured data after module completion.
  returned: when changed
  type: list
  sample: The configuration returned will always be in the same format of the parameters above.
commands:
  description: The set of commands pushed to the remote device
  returned: always
  type: list
  sample: ['interface GigabitEthernet0/0/0/2', 'l2transport l2protocol pvst tunnel']
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.argspec.l2_interfaces.l2_interfaces import (
    L2_InterfacesArgs,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.config.l2_interfaces.l2_interfaces import (
    L2_Interfaces,
)


def main():
    """
    Main entry point for module execution
    :returns: the result form module invocation
    """
    required_if = [
        ("state", "merged", ("config",)),
        ("state", "replaced", ("config",)),
        ("state", "rendered", ("config",)),
        ("state", "overridden", ("config",)),
        ("state", "parsed", ("running_config",)),
    ]
    mutually_exclusive = [("config", "running_config")]
    module = AnsibleModule(
        argument_spec=L2_InterfacesArgs.argument_spec,
        required_if=required_if,
        supports_check_mode=True,
        mutually_exclusive=mutually_exclusive,
    )

    result = L2_Interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Abhijeet Kasurde <akasurde@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = r'''
---
module: vmware_vm_vm_drs_rule
short_description: Configure VMware DRS Affinity rule for virtual machine in given cluster
description:
- This module can be used to configure VMware DRS Affinity rule for virtual machine in given cluster.
author:
- Abhijeet Kasurde (@Akasurde)
notes:
- Tested on vSphere 6.5
requirements:
- "python >= 2.6"
- PyVmomi
options:
  cluster_name:
    description:
    - Desired cluster name where virtual machines are present for the DRS rule.
    required: True
    type: str
  vms:
    description:
    - List of virtual machines name for which DRS rule needs to be applied.
    - Required if C(state) is set to C(present).
    type: list
    elements: str
  drs_rule_name:
    description:
    - The name of the DRS rule to manage.
    required: True
    type: str
  enabled:
    description:
    - If set to C(True), the DRS rule will be enabled.
    - Effective only if C(state) is set to C(present).
    default: False
    type: bool
  mandatory:
    description:
    - If set to C(True), the DRS rule will be mandatory.
    - Effective only if C(state) is set to C(present).
    default: False
    type: bool
  affinity_rule:
    description:
    - If set to C(True), the DRS rule will be an Affinity rule.
    - If set to C(False), the DRS rule will be an Anti-Affinity rule.
    - Effective only if C(state) is set to C(present).
    default: True
    type: bool
  state:
    description:
    - If set to C(present), then the DRS rule is created if not present.
    - If set to C(present), then the DRS rule is already present, it updates to the given configurations.
    - If set to C(absent), then the DRS rule is deleted if present.
    required: False
    default: present
    choices: [ present, absent ]
    type: str
extends_documentation_fragment:
- community.vmware.vmware.documentation

'''

EXAMPLES = r'''
- name: Create DRS Affinity Rule for VM-VM
  community.vmware.vmware_vm_vm_drs_rule:
    hostname: "{{ esxi_server }}"
    username: "{{ esxi_username }}"
    password: "{{ esxi_password }}"
    cluster_name: "{{ cluster_name }}"
    validate_certs: no
    vms:
        - vm1
        - vm2
    drs_rule_name: vm1-vm2-affinity-rule-001
    enabled: True
    mandatory: True
    affinity_rule: True
  delegate_to: localhost

- name: Create DRS Anti-Affinity Rule for VM-VM
  community.vmware.vmware_vm_vm_drs_rule:
    hostname: "{{ esxi_server }}"
    username: "{{ esxi_username }}"
    password: "{{ esxi_password }}"
    cluster_name: "{{ cluster_name }}"
    validate_certs: no
    enabled: True
    vms:
        - vm1
        - vm2
    drs_rule_name: vm1-vm2-affinity-rule-001
    mandatory: True
    affinity_rule: False
  delegate_to: localhost

- name: Delete DRS Affinity Rule for VM-VM
  community.vmware.vmware_vm_vm_drs_rule:
    hostname: "{{ esxi_server }}"
    username: "{{ esxi_username }}"
    password: "{{ esxi_password }}"
    cluster_name: "{{ cluster_name }}"
    validate_certs: no
    drs_rule_name: vm1-vm2-affinity-rule-001
    state: absent
  delegate_to: localhost
'''

RETURN = r'''
result:
    description: metadata about DRS VM and VM rule
    returned: when state is present
    type: dict
    sample: {
            "rule_enabled": false,
            "rule_key": 20,
            "rule_mandatory": true,
            "rule_name": "drs_rule_0014",
            "rule_uuid": "525f3bc0-253f-825a-418e-2ec93bffc9ae",
            "rule_vms": [
                "VM_65",
                "VM_146"
            ]
        }
'''

try:
    from pyVmomi import vim, vmodl
except ImportError:
    pass

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils._text import to_native
from ansible_collections.community.vmware.plugins.module_utils.vmware import (
    PyVmomi, vmware_argument_spec, wait_for_task,
    find_vm_by_id, find_cluster_by_name)


class VmwareDrs(PyVmomi):
    def __init__(self, module):
        super(VmwareDrs, self).__init__(module)
        self.vm_list = module.params['vms']
        self.cluster_name = module.params['cluster_name']
        self.rule_name = module.params['drs_rule_name']
        self.enabled = module.params['enabled']
        self.mandatory = module.params['mandatory']
        self.affinity_rule = module.params['affinity_rule']
        self.state = module.params['state']

        # Sanity check for cluster
        self.cluster_obj = find_cluster_by_name(content=self.content,
                                                cluster_name=self.cluster_name)
        if self.cluster_obj is None:
            self.module.fail_json(msg="Failed to find the cluster %s" % self.cluster_name)
        # Sanity check for virtual machines
        self.vm_obj_list = []
        if self.state == 'present':
            # Get list of VMs only if state is present
            self.vm_obj_list = self.get_all_vms_info()

    # Getter
    def get_all_vms_info(self, vms_list=None):
        """
        Get all VM objects using name from given cluster
        Args:
            vms_list: List of VM names

        Returns: List of VM managed objects

        """
        vm_obj_list = []
        if vms_list is None:
            vms_list = self.vm_list

        for vm_name in vms_list:
            vm_obj = find_vm_by_id(content=self.content, vm_id=vm_name,
                                   vm_id_type='vm_name', cluster=self.cluster_obj)
            if vm_obj is None:
                self.module.fail_json(msg="Failed to find the virtual machine %s "
                                          "in given cluster %s" % (vm_name,
                                                                   self.cluster_name))
            vm_obj_list.append(vm_obj)
        return vm_obj_list

    def get_rule_key_by_name(self, cluster_obj=None, rule_name=None):
        """
        Get a specific DRS rule key by name
        Args:
            rule_name: Name of rule
            cluster_obj: Cluster managed object

        Returns: Rule Object if found or None

        """
        if cluster_obj is None:
            cluster_obj = self.cluster_obj

        if rule_name:
            rules_list = [rule for rule in cluster_obj.configuration.rule if rule.name == rule_name]
            if rules_list:
                return rules_list[0]
        # No rule found
        return None

    @staticmethod
    def normalize_rule_spec(rule_obj=None):
        """
        Return human readable rule spec
        Args:
            rule_obj: Rule managed object

        Returns: Dictionary with Rule info

        """
        if rule_obj is None:
            return {}
        return dict(rule_key=rule_obj.key,
                    rule_enabled=rule_obj.enabled,
                    rule_name=rule_obj.name,
                    rule_mandatory=rule_obj.mandatory,
                    rule_uuid=rule_obj.ruleUuid,
                    rule_vms=[vm.name for vm in rule_obj.vm],
                    rule_affinity=True if isinstance(rule_obj, vim.cluster.AffinityRuleSpec) else False,
                    )

    # Create
    def create(self):
        """
        Create a DRS rule if rule does not exist
        """
        rule_obj = self.get_rule_key_by_name(rule_name=self.rule_name)
        if rule_obj is not None:
            existing_rule = self.normalize_rule_spec(rule_obj=rule_obj)
            if ((sorted(existing_rule['rule_vms']) == sorted(self.vm_list))
                    and (existing_rule['rule_enabled'] == self.enabled)
                    and (existing_rule['rule_mandatory'] == self.mandatory)
                    and (existing_rule['rule_affinity'] == self.affinity_rule)):
                self.module.exit_json(changed=False, result=existing_rule, msg="Rule already exists with the same configuration")
            else:
                changed, result = self.update_rule_spec(rule_obj)
                return changed, result
        else:
            changed, result = self.create_rule_spec()
            return changed, result

    def create_rule_spec(self):
        """
        Create DRS rule
        """
        changed = False
        if self.affinity_rule:
            rule = vim.cluster.AffinityRuleSpec()
        else:
            rule = vim.cluster.AntiAffinityRuleSpec()

        rule.vm = self.vm_obj_list
        rule.enabled = self.enabled
        rule.mandatory = self.mandatory
        rule.name = self.rule_name

        rule_spec = vim.cluster.RuleSpec(info=rule, operation='add')
        config_spec = vim.cluster.ConfigSpecEx(rulesSpec=[rule_spec])

        try:
            task = self.cluster_obj.ReconfigureEx(config_spec, modify=True)
            changed, result = wait_for_task(task)
        except vmodl.fault.InvalidRequest as e:
            result = to_native(e.msg)
        except Exception as e:
            result = to_native(e)

        if changed:
            rule_obj = self.get_rule_key_by_name(rule_name=self.rule_name)
            result = self.normalize_rule_spec(rule_obj)

        return changed, result

    def update_rule_spec(self, rule_obj=None):
        """
        Update DRS rule
        """
        changed = False

        rule_obj.vm = self.vm_obj_list

        if (rule_obj.mandatory != self.mandatory):
            rule_obj.mandatory = self.mandatory

        if (rule_obj.enabled != self.enabled):
            rule_obj.enabled = self.enabled

        rule_spec = vim.cluster.RuleSpec(info=rule_obj, operation='edit')
        config_spec = vim.cluster.ConfigSpec(rulesSpec=[rule_spec])

        try:
            task = self.cluster_obj.ReconfigureCluster_Task(config_spec, modify=True)
            changed, result = wait_for_task(task)
        except vmodl.fault.InvalidRequest as e:
            result = to_native(e.msg)
        except Exception as e:
            result = to_native(e)

        if changed:
            rule_obj = self.get_rule_key_by_name(rule_name=self.rule_name)
            result = self.normalize_rule_spec(rule_obj)

        return changed, result

    # Delete
    def delete(self, rule_name=None):
        """
        Delete DRS rule using name
        """
        changed = False
        if rule_name is None:
            rule_name = self.rule_name

        rule = self.get_rule_key_by_name(rule_name=rule_name)
        if rule is not None:
            rule_key = int(rule.key)
            rule_spec = vim.cluster.RuleSpec(removeKey=rule_key, operation='remove')
            config_spec = vim.cluster.ConfigSpecEx(rulesSpec=[rule_spec])
            try:
                task = self.cluster_obj.ReconfigureEx(config_spec, modify=True)
                changed, result = wait_for_task(task)
            except vmodl.fault.InvalidRequest as e:
                result = to_native(e.msg)
            except Exception as e:
                result = to_native(e)
        else:
            result = 'No rule named %s exists' % self.rule_name
        return changed, result


def main():
    argument_spec = vmware_argument_spec()
    argument_spec.update(dict(
        state=dict(type='str', default='present', choices=['absent', 'present']),
        vms=dict(type='list', elements='str'),
        cluster_name=dict(type='str', required=True),
        drs_rule_name=dict(type='str', required=True),
        enabled=dict(type='bool', default=False),
        mandatory=dict(type='bool', default=False),
        affinity_rule=dict(type='bool', default=True),
    )
    )

    required_if = [
        ['state', 'present', ['vms']]
    ]
    module = AnsibleModule(argument_spec=argument_spec,
                           required_if=required_if,
                           supports_check_mode=True)

    results = dict(failed=False, changed=False)
    state = module.params['state']
    vm_drs = VmwareDrs(module)

    if state == 'present':
        # Add Rule
        if module.check_mode:
            results['changed'] = True
            module.exit_json(**results)
        changed, result = vm_drs.create()
        if changed:
            results['changed'] = changed
        else:
            results['failed'] = True
            results['msg'] = "Failed to create DRS rule %s" % vm_drs.rule_name
        results['result'] = result
    elif state == 'absent':
        # Delete Rule
        if module.check_mode:
            results['changed'] = True
            module.exit_json(**results)
        changed, result = vm_drs.delete()
        if changed:
            results['changed'] = changed
            results['msg'] = "DRS rule %s deleted successfully." % vm_drs.rule_name
        else:
            if "No rule named" in result:
                results['msg'] = result
                module.exit_json(**results)

            results['failed'] = True
            results['msg'] = "Failed to delete DRS rule %s" % vm_drs.rule_name
        results['result'] = result

    if results['changed']:
        module.exit_json(**results)
    if results['failed']:
        module.fail_json(**results)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# (c) 2017, Ansible by Red Hat, inc
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = """
module: net_logging
author: Ganesh Nalawade (@ganeshrn)
short_description: (deprecated, removed after 2022-06-01) Manage logging on network
  devices
description:
- This module provides declarative management of logging on network devices.
version_added: 1.0.0
deprecated:
  alternative: Use platform-specific "[netos]_logging" module
  why: Updated modules released with more functionality
  removed_at_date: '2022-06-01'
extends_documentation_fragment:
- ansible.netcommon.network_agnostic
options:
  dest:
    description:
    - Destination of the logs.
    choices:
    - console
    - host
  name:
    description:
    - If value of C(dest) is I(host) it indicates file-name the host name to be notified.
  facility:
    description:
    - Set logging facility.
  level:
    description:
    - Set logging severity levels.
  aggregate:
    description: List of logging definitions.
  purge:
    description:
    - Purge logging not defined in the I(aggregate) parameter.
    default: false
  state:
    description:
    - State of the logging configuration.
    default: present
    choices:
    - present
    - absent

"""

EXAMPLES = """
- name: configure console logging
  ansible.netcommon.net_logging:
    dest: console
    facility: any
    level: critical

- name: remove console logging configuration
  ansible.netcommon.net_logging:
    dest: console
    state: absent

- name: configure host logging
  ansible.netcommon.net_logging:
    dest: host
    name: 192.0.2.1
    facility: kernel
    level: critical

- name: Configure file logging using aggregate
  ansible.netcommon.net_logging:
    dest: file
    aggregate:
    - name: test-1
      facility: pfe
      level: critical
    - name: test-2
      facility: kernel
      level: emergency
- name: Delete file logging using aggregate
  ansible.netcommon.net_logging:
    dest: file
    aggregate:
    - name: test-1
      facility: pfe
      level: critical
    - name: test-2
      facility: kernel
      level: emergency
    state: absent
"""

RETURN = """
commands:
  description: The list of configuration mode commands to send to the device
  returned: always, except for the platforms that use Netconf transport to manage the device.
  type: list
  sample:
    - logging console critical
"""



#!/usr/bin/python
#  -*- coding: utf-8 -*-
#  Copyright: (c) 2019, Ansible Project
#  Copyright: (c) 2019, Diane Wang <dianew@vmware.com>
#  GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: vmware_guest_screenshot
short_description: Create a screenshot of the Virtual Machine console.
description:
    - This module is used to take screenshot of the given virtual machine when virtual machine is powered on.
    - All parameters and VMware object names are case sensitive.
author:
    - Diane Wang (@Tomorrow9) <dianew@vmware.com>
notes:
    - Tested on vSphere 6.5 and 6.7
requirements:
    - "python >= 2.6"
    - PyVmomi
options:
   name:
     description:
     - Name of the virtual machine.
     - This is a required parameter, if parameter C(uuid) or C(moid) is not supplied.
     type: str
   uuid:
     description:
     - UUID of the instance to gather facts if known, this is VMware's unique identifier.
     - This is a required parameter, if parameter C(name) or C(moid) is not supplied.
     type: str
   moid:
     description:
     - Managed Object ID of the instance to manage if known, this is a unique identifier only within a single vCenter instance.
     - This is required if C(name) or C(uuid) is not supplied.
     type: str
   folder:
     description:
     - Destination folder, absolute or relative path to find an existing guest.
     - This is a required parameter, only if multiple VMs are found with same name.
     - The folder should include the datacenter. ESXi server's datacenter is ha-datacenter.
     - 'Examples:'
     - '   folder: /ha-datacenter/vm'
     - '   folder: ha-datacenter/vm'
     - '   folder: /datacenter1/vm'
     - '   folder: datacenter1/vm'
     - '   folder: /datacenter1/vm/folder1'
     - '   folder: datacenter1/vm/folder1'
     - '   folder: /folder1/datacenter1/vm'
     - '   folder: folder1/datacenter1/vm'
     - '   folder: /folder1/datacenter1/vm/folder2'
     type: str
   cluster:
     description:
     - The name of cluster where the virtual machine is running.
     - This is a required parameter, if C(esxi_hostname) is not set.
     - C(esxi_hostname) and C(cluster) are mutually exclusive parameters.
     type: str
   esxi_hostname:
     description:
     - The ESXi hostname where the virtual machine is running.
     - This is a required parameter, if C(cluster) is not set.
     - C(esxi_hostname) and C(cluster) are mutually exclusive parameters.
     type: str
   datacenter:
     description:
     - The datacenter name to which virtual machine belongs to.
     type: str
   local_path:
     description:
     - 'If C(local_path) is not set, the created screenshot file will be kept in the directory of the virtual machine
       on ESXi host. If C(local_path) is set to a valid path on local machine, then the screenshot file will be
       downloaded from ESXi host to the local directory.'
     - 'If not download screenshot file to local machine, you can open it through the returned file URL in screenshot
       facts manually.'
     type: path
extends_documentation_fragment:
- community.vmware.vmware.documentation

'''

EXAMPLES = '''
- name: take a screenshot of the virtual machine console
  community.vmware.vmware_guest_screenshot:
    validate_certs: no
    hostname: "{{ vcenter_hostname }}"
    username: "{{ vcenter_username }}"
    password: "{{ vcenter_password }}"
    datacenter: "{{ datacenter_name }}"
    folder: "{{ folder_name }}"
    name: "{{ vm_name }}"
    local_path: "/tmp/"
  delegate_to: localhost
  register: take_screenshot

- name: Take a screenshot of the virtual machine console using MoID
  community.vmware.vmware_guest_screenshot:
    validate_certs: no
    hostname: "{{ vcenter_hostname }}"
    username: "{{ vcenter_username }}"
    password: "{{ vcenter_password }}"
    datacenter: "{{ datacenter_name }}"
    folder: "{{ folder_name }}"
    moid: vm-42
    local_path: "/tmp/"
  delegate_to: localhost
  register: take_screenshot
'''

RETURN = """
screenshot_info:
    description: display the facts of captured virtual machine screenshot file
    returned: always
    type: dict
    sample: {
            "virtual_machine": "test_vm",
            "screenshot_file": "[datastore0] test_vm/test_vm-1.png",
            "task_start_time": "2019-05-25T10:35:04.215016Z",
            "task_complete_time": "2019-05-25T10:35:04.412622Z",
            "result": "success",
            "screenshot_file_url": "https://test_vcenter/folder/test_vm/test_vm-1.png?dcPath=test-dc&dsName=datastore0",
            "download_local_path": "/tmp/",
            "download_file_size": 2367,
    }
"""

try:
    from pyVmomi import vim, vmodl
except ImportError:
    pass

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.six.moves.urllib.parse import urlencode, quote
from ansible.module_utils._text import to_native
from ansible.module_utils.urls import open_url
from ansible_collections.community.vmware.plugins.module_utils.vmware import PyVmomi, vmware_argument_spec, wait_for_task, get_parent_datacenter
import os


class PyVmomiHelper(PyVmomi):
    def __init__(self, module):
        super(PyVmomiHelper, self).__init__(module)
        self.change_detected = False

    def generate_http_access_url(self, file_path):
        # e.g., file_path is like this format: [datastore0] test_vm/test_vm-1.png
        # from file_path generate URL
        url_path = None
        if not file_path:
            return url_path

        path = "/folder/%s" % quote(file_path.split()[1])
        params = dict(dsName=file_path.split()[0].strip('[]'))
        if not self.is_vcenter():
            datacenter = 'ha-datacenter'
        else:
            datacenter = get_parent_datacenter(self.current_vm_obj).name.replace('&', '%26')
        params['dcPath'] = datacenter
        url_path = "https://%s%s?%s" % (self.params['hostname'], path, urlencode(params))

        return url_path

    def download_screenshot_file(self, file_url, local_file_path, file_name):
        response = None
        download_size = 0
        # file is downloaded as local_file_name when specified, or use original file name
        if local_file_path.endswith('.png'):
            local_file_name = local_file_path.split('/')[-1]
            local_file_path = local_file_path.rsplit('/', 1)[0]
        else:
            local_file_name = file_name
        if not os.path.exists(local_file_path):
            try:
                os.makedirs(local_file_path)
            except OSError as err:
                self.module.fail_json(msg="Exception caught when create folder %s on local machine, with error %s"
                                          % (local_file_path, to_native(err)))
        local_file = os.path.join(local_file_path, local_file_name)
        with open(local_file, 'wb') as handle:
            try:
                response = open_url(file_url, url_username=self.params.get('username'),
                                    url_password=self.params.get('password'), validate_certs=False)
            except Exception as err:
                self.module.fail_json(msg="Download screenshot file from URL %s, failed due to %s" % (file_url, to_native(err)))
            if not response or response.getcode() >= 400:
                self.module.fail_json(msg="Download screenshot file from URL %s, failed with response %s, response code %s"
                                          % (file_url, response, response.getcode()))
            bytes_read = response.read(2 ** 20)
            while bytes_read:
                handle.write(bytes_read)
                handle.flush()
                os.fsync(handle.fileno())
                download_size += len(bytes_read)
                bytes_read = response.read(2 ** 20)

        return download_size

    def get_screenshot_facts(self, task_info, file_url, file_size):
        screenshot_facts = dict()
        if task_info is not None:
            screenshot_facts = dict(
                virtual_machine=task_info.entityName,
                screenshot_file=task_info.result,
                task_start_time=task_info.startTime,
                task_complete_time=task_info.completeTime,
                result=task_info.state,
                screenshot_file_url=file_url,
                download_local_path=self.params.get('local_path'),
                download_file_size=file_size,
            )

        return screenshot_facts

    def take_vm_screenshot(self):
        if self.current_vm_obj.runtime.powerState != vim.VirtualMachinePowerState.poweredOn:
            self.module.fail_json(msg="VM is %s, valid power state is poweredOn." % self.current_vm_obj.runtime.powerState)
        try:
            task = self.current_vm_obj.CreateScreenshot_Task()
            wait_for_task(task)
        except vim.fault.FileFault as e:
            self.module.fail_json(msg="Failed to create screenshot due to errors when creating or accessing one or more"
                                      " files needed for this operation, %s" % to_native(e.msg))
        except vim.fault.InvalidState as e:
            self.module.fail_json(msg="Failed to create screenshot due to VM is not ready to respond to such requests,"
                                      " %s" % to_native(e.msg))
        except vmodl.RuntimeFault as e:
            self.module.fail_json(msg="Failed to create screenshot due to runtime fault, %s," % to_native(e.msg))
        except vim.fault.TaskInProgress as e:
            self.module.fail_json(msg="Failed to create screenshot due to VM is busy, %s" % to_native(e.msg))

        if task.info.state == 'error':
            return {'changed': self.change_detected, 'failed': True, 'msg': task.info.error.msg}
        else:
            download_file_size = None
            self.change_detected = True
            file_url = self.generate_http_access_url(task.info.result)
            if self.params.get('local_path'):
                if file_url:
                    download_file_size = self.download_screenshot_file(file_url=file_url,
                                                                       local_file_path=self.params['local_path'],
                                                                       file_name=task.info.result.split('/')[-1])
            screenshot_facts = self.get_screenshot_facts(task.info, file_url, download_file_size)
            return {'changed': self.change_detected, 'failed': False, 'screenshot_info': screenshot_facts}


def main():
    argument_spec = vmware_argument_spec()
    argument_spec.update(
        name=dict(type='str'),
        uuid=dict(type='str'),
        moid=dict(type='str'),
        folder=dict(type='str'),
        datacenter=dict(type='str'),
        esxi_hostname=dict(type='str'),
        cluster=dict(type='str'),
        local_path=dict(type='path'),
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        required_one_of=[
            ['name', 'uuid', 'moid']
        ]
    )
    pyv = PyVmomiHelper(module)
    vm = pyv.get_vm()
    if not vm:
        vm_id = (module.params.get('uuid') or module.params.get('name') or module.params.get('moid'))
        module.fail_json(msg='Unable to find the specified virtual machine : %s' % vm_id)

    result = pyv.take_vm_screenshot()
    if result['failed']:
        module.fail_json(**result)
    else:
        module.exit_json(**result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# Copyright: (c) 2018, Bojan Vitnik <bvitnik@mainstream.rs>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type


class ModuleDocFragment(object):
    # Common parameters for XenServer modules
    DOCUMENTATION = r'''
options:
  hostname:
    description:
    - The hostname or IP address of the XenServer host or XenServer pool master.
    - If the value is not specified in the task, the value of environment variable C(XENSERVER_HOST) will be used instead.
    type: str
    default: localhost
    aliases: [ host, pool ]
  username:
    description:
    - The username to use for connecting to XenServer.
    - If the value is not specified in the task, the value of environment variable C(XENSERVER_USER) will be used instead.
    type: str
    default: root
    aliases: [ admin, user ]
  password:
    description:
    - The password to use for connecting to XenServer.
    - If the value is not specified in the task, the value of environment variable C(XENSERVER_PASSWORD) will be used instead.
    type: str
    aliases: [ pass, pwd ]
  validate_certs:
    description:
    - Allows connection when SSL certificates are not valid. Set to C(false) when certificates are not trusted.
    - If the value is not specified in the task, the value of environment variable C(XENSERVER_VALIDATE_CERTS) will be used instead.
    type: bool
    default: yes
'''



#!/usr/bin/python
#
# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = """
module: iosxr_command
author: Ricardo Carrillo Cruz (@rcarrillocruz)
short_description: Run commands on remote devices running Cisco IOS XR
description:
- Sends arbitrary commands to an IOS XR node and returns the results read from the
  device. This module includes an argument that will cause the module to wait for
  a specific condition before returning or timing out if the condition is not met.
- This module does not support running commands in configuration mode. Please use
  M(iosxr_config) to configure iosxr devices.
version_added: 1.0.0
extends_documentation_fragment:
- cisco.iosxr.iosxr
notes:
- Make sure the user has been authorized to execute commands terminal length 0, terminal
  width 512 and terminal exec prompt no-timestamp.
- This module works with C(network_cli). See L(the IOS-XR Platform Options,../network/user_guide/platform_iosxr.html).
- This module does not support C(netconf) connection.
- Tested against IOS XR 6.1.3
options:
  commands:
    description:
    - List of commands to send to the remote iosxr device over the configured provider.
      The resulting output from the command is returned. If the I(wait_for) argument
      is provided, the module is not returned until the condition is satisfied or
      the number of retries has expired.
    - If a command sent to the device requires answering a prompt, it is possible to pass
      a dict containing command, answer and prompt. Common answers are 'y' or "\\r"
      (carriage return, must be double quotes). See examples
    type: list
    elements: raw
    required: true
  wait_for:
    description:
    - List of conditions to evaluate against the output of the command. The task will
      wait for each condition to be true before moving forward. If the conditional
      is not true within the configured number of retries, the task fails. See examples.
    aliases:
    - waitfor
    type: list
    elements: str
  match:
    description:
    - The I(match) argument is used in conjunction with the I(wait_for) argument to
      specify the match policy.  Valid values are C(all) or C(any).  If the value
      is set to C(all) then all conditionals in the wait_for must be satisfied.  If
      the value is set to C(any) then only one of the values must be satisfied.
    default: all
    choices:
    - any
    - all
    type: str
  retries:
    description:
    - Specifies the number of retries a command should by tried before it is considered
      failed. The command is run on the target device every retry and evaluated against
      the I(wait_for) conditions.
    default: 10
    type: int
  interval:
    description:
    - Configures the interval in seconds to wait between retries of the command. If
      the command does not pass the specified conditions, the interval indicates how
      long to wait before trying the command again.
    default: 1
    type: int
"""

EXAMPLES = """
- name: run show version on remote devices
  cisco.iosxr.iosxr_command:
    commands: show version

- name: run show version and check to see if output contains iosxr
  cisco.iosxr.iosxr_command:
    commands: show version
    wait_for: result[0] contains IOS-XR

- name: run multiple commands on remote nodes
  cisco.iosxr.iosxr_command:
    commands:
    - show version
    - show interfaces
    - {command: example command that prompts, prompt: expected prompt, answer: yes}

- name: run multiple commands and evaluate the output
  cisco.iosxr.iosxr_command:
    commands:
    - show version
    - show interfaces
    wait_for:
    - result[0] contains IOS-XR
    - result[1] contains Loopback0
"""

RETURN = """
stdout:
  description: The set of responses from the commands
  returned: always apart from low level errors (such as action plugin)
  type: list
  sample: ['...', '...']
stdout_lines:
  description: The value of stdout split into a list
  returned: always apart from low level errors (such as action plugin)
  type: list
  sample: [['...', '...'], ['...'], ['...']]
failed_conditions:
  description: The list of conditionals that have failed
  returned: failed
  type: list
  sample: ['...', '...']
"""
import time

from ansible.module_utils._text import to_text
from ansible.module_utils.basic import AnsibleModule
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.parsing import (
    Conditional,
)
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.utils import (
    to_lines,
)
from ansible_collections.cisco.iosxr.plugins.module_utils.network.iosxr.iosxr import (
    run_commands,
    iosxr_argument_spec,
)


def parse_commands(module, warnings):
    commands = module.params["commands"]
    for item in list(commands):
        try:
            command = item["command"]
        except Exception:
            command = item
        if module.check_mode and not command.startswith("show"):
            warnings.append(
                "Only show commands are supported when using check mode, not "
                "executing %s" % command
            )
            commands.remove(item)

    return commands


def main():
    argument_spec = dict(
        commands=dict(type="list", required=True, elements="raw"),
        wait_for=dict(type="list", aliases=["waitfor"], elements="str"),
        match=dict(default="all", choices=["all", "any"]),
        retries=dict(default=10, type="int"),
        interval=dict(default=1, type="int"),
    )

    argument_spec.update(iosxr_argument_spec)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )

    warnings = list()
    result = {"changed": False, "warnings": warnings}
    commands = parse_commands(module, warnings)
    wait_for = module.params["wait_for"] or list()

    try:
        conditionals = [Conditional(c) for c in wait_for]
    except AttributeError as exc:
        module.fail_json(msg=to_text(exc))

    retries = module.params["retries"]
    interval = module.params["interval"]
    match = module.params["match"]

    while retries > 0:
        responses = run_commands(module, commands)

        for item in list(conditionals):
            if item(responses):
                if match == "any":
                    conditionals = list()
                    break
                conditionals.remove(item)

        if not conditionals:
            break

        time.sleep(interval)
        retries -= 1

    if conditionals:
        failed_conditions = [item.raw for item in conditionals]
        msg = "One or more conditional statements have not been satisfied"
        module.fail_json(msg=msg, failed_conditions=failed_conditions)

    result.update(
        {"stdout": responses, "stdout_lines": list(to_lines(responses))}
    )

    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = """
module: nxos_aaa_server
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: Manages AAA server global configuration.
description:
- Manages AAA server global configuration
version_added: 1.0.0
author:
- Jason Edelman (@jedelman8)
notes:
- Tested against NXOSv 7.3.(0)D1(1) on VIRL
- The server_type parameter is always required.
- If encrypt_type is not supplied, the global AAA server key will be stored as encrypted
  (type 7).
- Changes to the global AAA server key with encrypt_type=0 are not idempotent.
- state=default will set the supplied parameters to their default values. The parameters
  that you want to default must also be set to default. If global_key=default, the
  global key will be removed.
options:
  server_type:
    description:
    - The server type is either radius or tacacs.
    required: true
    choices:
    - radius
    - tacacs
    type: str
  global_key:
    description:
    - Global AAA shared secret or keyword 'default'.
    type: str
  encrypt_type:
    description:
    - The state of encryption applied to the entered global key. O clear text, 7 encrypted.
      Type-6 encryption is not supported.
    choices:
    - '0'
    - '7'
    type: str
  deadtime:
    description:
    - Duration for which a non-reachable AAA server is skipped, in minutes or keyword
      'default. Range is 1-1440. Device default is 0.
    type: str
  server_timeout:
    description:
    - Global AAA server timeout period, in seconds or keyword 'default. Range is 1-60.
      Device default is 5.
    type: str
  directed_request:
    description:
    - Enables direct authentication requests to AAA server or keyword 'default' Device
      default is disabled.
    choices:
    - enabled
    - disabled
    - default
    type: str
  state:
    description:
    - Manage the state of the resource.
    default: present
    choices:
    - present
    - default
    type: str
"""

EXAMPLES = """
# Radius Server Basic settings
- name: Radius Server Basic settings
  cisco.nxos.nxos_aaa_server:
    server_type: radius
    server_timeout: 9
    deadtime: 20
    directed_request: enabled

# Tacacs Server Basic settings
- name: Tacacs Server Basic settings
  cisco.nxos.nxos_aaa_server:
    server_type: tacacs
    server_timeout: 8
    deadtime: 19
    directed_request: disabled

# Setting Global Key
- name: AAA Server Global Key
  cisco.nxos.nxos_aaa_server:
    server_type: radius
    global_key: test_key
"""

RETURN = """
commands:
    description: command sent to the device
    returned: always
    type: list
    sample: ["radius-server deadtime 22", "radius-server timeout 11",
             "radius-server directed-request"]
"""
import re

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    load_config,
    run_commands,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    nxos_argument_spec,
)
from ansible.module_utils.basic import AnsibleModule


PARAM_TO_DEFAULT_KEYMAP = {
    "server_timeout": "5",
    "deadtime": "0",
    "directed_request": "disabled",
}


def execute_show_command(command, module):
    command = {"command": command, "output": "text"}

    return run_commands(module, command)


def flatten_list(command_lists):
    flat_command_list = []
    for command in command_lists:
        if isinstance(command, list):
            flat_command_list.extend(command)
        else:
            flat_command_list.append(command)
    return flat_command_list


def get_aaa_server_info(server_type, module):
    aaa_server_info = {}
    server_command = "show {0}-server".format(server_type)
    request_command = "show {0}-server directed-request".format(server_type)
    global_key_command = "show run | sec {0}".format(server_type)
    aaa_regex = r".*{0}-server\skey\s\d\s+(?P<key>\S+).*".format(server_type)

    server_body = execute_show_command(server_command, module)[0]

    split_server = server_body.splitlines()

    for line in split_server:
        if line.startswith("timeout"):
            aaa_server_info["server_timeout"] = line.split(":")[1]

        elif line.startswith("deadtime"):
            aaa_server_info["deadtime"] = line.split(":")[1]

    request_body = execute_show_command(request_command, module)[0]

    if bool(request_body):
        aaa_server_info["directed_request"] = request_body.replace("\n", "")
    else:
        aaa_server_info["directed_request"] = "disabled"

    key_body = execute_show_command(global_key_command, module)[0]

    try:
        match_global_key = re.match(aaa_regex, key_body, re.DOTALL)
        group_key = match_global_key.groupdict()
        aaa_server_info["global_key"] = group_key["key"].replace('"', "")
    except (AttributeError, TypeError):
        aaa_server_info["global_key"] = None

    return aaa_server_info


def config_aaa_server(params, server_type):
    cmds = []

    deadtime = params.get("deadtime")
    server_timeout = params.get("server_timeout")
    directed_request = params.get("directed_request")
    encrypt_type = params.get("encrypt_type", "7")
    global_key = params.get("global_key")

    if deadtime is not None:
        cmds.append("{0}-server deadtime {1}".format(server_type, deadtime))

    if server_timeout is not None:
        cmds.append(
            "{0}-server timeout {1}".format(server_type, server_timeout)
        )

    if directed_request is not None:
        if directed_request == "enabled":
            cmds.append("{0}-server directed-request".format(server_type))
        elif directed_request == "disabled":
            cmds.append("no {0}-server directed-request".format(server_type))

    if global_key is not None:
        cmds.append(
            "{0}-server key {1} {2}".format(
                server_type, encrypt_type, global_key
            )
        )

    return cmds


def default_aaa_server(existing, params, server_type):
    cmds = []

    deadtime = params.get("deadtime")
    server_timeout = params.get("server_timeout")
    directed_request = params.get("directed_request")
    global_key = params.get("global_key")
    existing_key = existing.get("global_key")

    if (
        deadtime is not None
        and existing.get("deadtime") != PARAM_TO_DEFAULT_KEYMAP["deadtime"]
    ):
        cmds.append("no {0}-server deadtime 1".format(server_type))

    if (
        server_timeout is not None
        and existing.get("server_timeout")
        != PARAM_TO_DEFAULT_KEYMAP["server_timeout"]
    ):
        cmds.append("no {0}-server timeout 1".format(server_type))

    if (
        directed_request is not None
        and existing.get("directed_request")
        != PARAM_TO_DEFAULT_KEYMAP["directed_request"]
    ):
        cmds.append("no {0}-server directed-request".format(server_type))

    if global_key is not None and existing_key is not None:
        cmds.append(
            "no {0}-server key 7 {1}".format(server_type, existing_key)
        )

    return cmds


def main():
    argument_spec = dict(
        server_type=dict(
            type="str", choices=["radius", "tacacs"], required=True
        ),
        global_key=dict(type="str"),
        encrypt_type=dict(type="str", choices=["0", "7"]),
        deadtime=dict(type="str"),
        server_timeout=dict(type="str"),
        directed_request=dict(
            type="str", choices=["enabled", "disabled", "default"]
        ),
        state=dict(choices=["default", "present"], default="present"),
    )

    argument_spec.update(nxos_argument_spec)

    module = AnsibleModule(
        argument_spec=argument_spec, supports_check_mode=True
    )

    warnings = list()
    results = {"changed": False, "commands": [], "warnings": warnings}

    server_type = module.params["server_type"]
    global_key = module.params["global_key"]
    encrypt_type = module.params["encrypt_type"]
    deadtime = module.params["deadtime"]
    server_timeout = module.params["server_timeout"]
    directed_request = module.params["directed_request"]
    state = module.params["state"]

    if encrypt_type and not global_key:
        module.fail_json(msg="encrypt_type must be used with global_key.")

    args = dict(
        server_type=server_type,
        global_key=global_key,
        encrypt_type=encrypt_type,
        deadtime=deadtime,
        server_timeout=server_timeout,
        directed_request=directed_request,
    )

    proposed = dict((k, v) for k, v in args.items() if v is not None)

    existing = get_aaa_server_info(server_type, module)

    commands = []
    if state == "present":
        if deadtime:
            try:
                if int(deadtime) < 0 or int(deadtime) > 1440:
                    raise ValueError
            except ValueError:
                module.fail_json(
                    msg="deadtime must be an integer between 0 and 1440"
                )

        if server_timeout:
            try:
                if int(server_timeout) < 1 or int(server_timeout) > 60:
                    raise ValueError
            except ValueError:
                module.fail_json(
                    msg="server_timeout must be an integer between 1 and 60"
                )

        delta = dict(set(proposed.items()).difference(existing.items()))
        if delta:
            command = config_aaa_server(delta, server_type)
            if command:
                commands.append(command)

    elif state == "default":
        for key, value in proposed.items():
            if key != "server_type" and value != "default":
                module.fail_json(
                    msg='Parameters must be set to "default"'
                    "when state=default"
                )
        command = default_aaa_server(existing, proposed, server_type)
        if command:
            commands.append(command)

    cmds = flatten_list(commands)
    if cmds:
        results["changed"] = True
        if not module.check_mode:
            load_config(module, cmds)
        if "configure" in cmds:
            cmds.pop(0)
        results["commands"] = cmds

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# (c) 2014-2015, Epic Games, Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: circonus_annotation
short_description: create an annotation in circonus
description:
    - Create an annotation event with a given category, title and description. Optionally start, end or durations can be provided
author: "Nick Harring (@NickatEpic)"
requirements:
    - requests (either >= 2.0.0 for Python 3, or >= 1.0.0 for Python 2)
notes:
    - Check mode isn't supported.
options:
    api_key:
        description:
           - Circonus API key
        required: true
    category:
        description:
           - Annotation Category
        required: true
    description:
        description:
            - Description of annotation
        required: true
    title:
        description:
            - Title of annotation
        required: true
    start:
        description:
            - Unix timestamp of event start
        default: I(now)
    stop:
        description:
            - Unix timestamp of event end
        default: I(now) + I(duration)
    duration:
        description:
            - Duration in seconds of annotation
        default: 0
'''
EXAMPLES = '''
- name: Create a simple annotation event with a source, defaults to start and end time of now
  community.general.circonus_annotation:
    api_key: XXXXXXXXXXXXXXXXX
    title: App Config Change
    description: This is a detailed description of the config change
    category: This category groups like annotations

- name: Create an annotation with a duration of 5 minutes and a default start time of now
  community.general.circonus_annotation:
    api_key: XXXXXXXXXXXXXXXXX
    title: App Config Change
    description: This is a detailed description of the config change
    category: This category groups like annotations
    duration: 300

- name: Create an annotation with a start_time and end_time
  community.general.circonus_annotation:
    api_key: XXXXXXXXXXXXXXXXX
    title: App Config Change
    description: This is a detailed description of the config change
    category: This category groups like annotations
    start_time: 1395940006
    end_time: 1395954407
'''

RETURN = '''
annotation:
    description: details about the created annotation
    returned: success
    type: complex
    contains:
        _cid:
            description: annotation identifier
            returned: success
            type: str
            sample: /annotation/100000
        _created:
            description: creation timestamp
            returned: success
            type: int
            sample: 1502236928
        _last_modified:
            description: last modification timestamp
            returned: success
            type: int
            sample: 1502236928
        _last_modified_by:
            description: last modified by
            returned: success
            type: str
            sample: /user/1000
        category:
            description: category of the created annotation
            returned: success
            type: str
            sample: alerts
        title:
            description: title of the created annotation
            returned: success
            type: str
            sample: WARNING
        description:
            description: description of the created annotation
            returned: success
            type: str
            sample: Host is down.
        start:
            description: timestamp, since annotation applies
            returned: success
            type: int
            sample: Host is down.
        stop:
            description: timestamp, since annotation ends
            returned: success
            type: str
            sample: Host is down.
        rel_metrics:
            description: Array of metrics related to this annotation, each metrics is a string.
            returned: success
            type: list
            sample:
                - 54321_kbps
'''
import json
import time
import traceback
from distutils.version import LooseVersion

REQUESTS_IMP_ERR = None
try:
    import requests
    HAS_REQUESTS = True
except ImportError:
    REQUESTS_IMP_ERR = traceback.format_exc()
    HAS_REQUESTS = False

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.six import PY3
from ansible.module_utils._text import to_native


def check_requests_dep(module):
    """Check if an adequate requests version is available"""
    if not HAS_REQUESTS:
        module.fail_json(msg=missing_required_lib('requests'), exception=REQUESTS_IMP_ERR)
    else:
        required_version = '2.0.0' if PY3 else '1.0.0'
        if LooseVersion(requests.__version__) < LooseVersion(required_version):
            module.fail_json(msg="'requests' library version should be >= %s, found: %s." % (required_version, requests.__version__))


def post_annotation(annotation, api_key):
    ''' Takes annotation dict and api_key string'''
    base_url = 'https://api.circonus.com/v2'
    anootate_post_endpoint = '/annotation'
    resp = requests.post(base_url + anootate_post_endpoint,
                         headers=build_headers(api_key), data=json.dumps(annotation))
    resp.raise_for_status()
    return resp


def create_annotation(module):
    ''' Takes ansible module object '''
    annotation = {}
    duration = module.params['duration']
    if module.params['start'] is not None:
        start = module.params['start']
    else:
        start = int(time.time())
    if module.params['stop'] is not None:
        stop = module.params['stop']
    else:
        stop = int(time.time()) + duration
    annotation['start'] = start
    annotation['stop'] = stop
    annotation['category'] = module.params['category']
    annotation['description'] = module.params['description']
    annotation['title'] = module.params['title']
    return annotation


def build_headers(api_token):
    '''Takes api token, returns headers with it included.'''
    headers = {'X-Circonus-App-Name': 'ansible',
               'Host': 'api.circonus.com', 'X-Circonus-Auth-Token': api_token,
               'Accept': 'application/json'}
    return headers


def main():
    '''Main function, dispatches logic'''
    module = AnsibleModule(
        argument_spec=dict(
            start=dict(type='int'),
            stop=dict(type='int'),
            category=dict(required=True),
            title=dict(required=True),
            description=dict(required=True),
            duration=dict(default=0, type='int'),
            api_key=dict(required=True, no_log=True)
        )
    )

    check_requests_dep(module)

    annotation = create_annotation(module)
    try:
        resp = post_annotation(annotation, module.params['api_key'])
    except requests.exceptions.RequestException as e:
        module.fail_json(msg='Request Failed', reason=to_native(e), exception=traceback.format_exc())
    module.exit_json(changed=True, annotation=resp.json())


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (C) 2019 Junyi Yi (@JunyiYi)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# ----------------------------------------------------------------------------
#
#     ***     AUTO GENERATED CODE    ***    AUTO GENERATED CODE     ***
#
# ----------------------------------------------------------------------------
#
#     This file is automatically generated by Magic Modules and manual
#     changes will be clobbered when the file is regenerated.
#
#
# ----------------------------------------------------------------------------

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: azure_rm_batchaccount
version_added: "2.9"
short_description: Manages a Batch Account on Azure
description:
    - Create, update and delete instance of Azure Batch Account.

options:
    resource_group:
        description:
            - The name of the resource group in which to create the Batch Account.
        required: true
        type: str
    name:
        description:
            - The name of the Batch Account.
        required: true
        type: str
    location:
        description:
            - Specifies the supported Azure location where the resource exists.
        type: str
    auto_storage_account:
        description:
            - Existing storage account with which to associate the Batch Account.
            - It can be the storage account name which is in the same resource group.
            - It can be the storage account ID. Fox example "/subscriptions/{subscription_id}/resourceGroups/
              {resource_group}/providers/Microsoft.Storage/storageAccounts/{name}".
            - It can be a dict which contains I(name) and I(resource_group) of the storage account.
    key_vault:
        description:
            - Existing key vault with which to associate the Batch Account.
            - It can be the key vault name which is in the same resource group.
            - It can be the key vault ID. For example "/subscriptions/{subscription_id}/resourceGroups/
              {resource_group}/providers/Microsoft.KeyVault/vaults/{name}".
            - It can be a dict which contains I(name) and I(resource_group) of the key vault.
    pool_allocation_mode:
        description:
            - The pool acclocation mode of the Batch Account.
        default: batch_service
        choices:
            - batch_service
            - user_subscription
        type: str
    state:
        description:
            - Assert the state of the Batch Account.
            - Use C(present) to create or update a Batch Account and C(absent) to delete it.
        default: present
        type: str
        choices:
            - present
            - absent

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Junyi Yi (@JunyiYi)
'''

EXAMPLES = '''
  - name: Create Batch Account
    azure_rm_batchaccount:
        resource_group: MyResGroup
        name: mybatchaccount
        location: eastus
        auto_storage_account:
          name: mystorageaccountname
        pool_allocation_mode: batch_service
'''

RETURN = '''
id:
    description:
        - The ID of the Batch account.
    returned: always
    type: str
    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Batch/batchAccounts/sampleacct"
account_endpoint:
    description:
        - The account endpoint used to interact with the Batch service.
    returned: always
    type: str
    sample: sampleacct.westus.batch.azure.com
'''

import time
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import normalize_location_name
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
from ansible.module_utils.common.dict_transformations import _snake_to_camel

try:
    from msrestazure.azure_exceptions import CloudError
    from msrest.polling import LROPoller
    from msrestazure.azure_operation import AzureOperationPoller
    from msrest.serialization import Model
    from azure.mgmt.batch import BatchManagementClient
except ImportError:
    # This is handled in azure_rm_common
    pass


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMBatchAccount(AzureRMModuleBaseExt):
    """Configuration class for an Azure RM Batch Account resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                required=True,
                type='str'
            ),
            name=dict(
                required=True,
                type='str'
            ),
            location=dict(
                type='str',
                updatable=False,
                disposition='/'
            ),
            auto_storage_account=dict(
                type='raw'
            ),
            key_vault=dict(
                type='raw',
                updatable=False,
                disposition='/'
            ),
            pool_allocation_mode=dict(
                default='batch_service',
                type='str',
                choices=['batch_service', 'user_subscription'],
                updatable=False,
                disposition='/'
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.name = None
        self.batch_account = dict()
        self.tags = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.to_do = Actions.NoAction

        super(AzureRMBatchAccount, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                  supports_check_mode=True,
                                                  supports_tags=True)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.batch_account[key] = kwargs[key]

        resource_group = self.get_resource_group(self.resource_group)
        if self.batch_account.get('location') is None:
            self.batch_account['location'] = resource_group.location
        if self.batch_account.get('auto_storage_account') is not None:
            self.batch_account['auto_storage'] = {
                'storage_account_id': self.normalize_resource_id(
                    self.batch_account.pop('auto_storage_account'),
                    '/subscriptions/{subscription_id}/resourceGroups/{resource_group}/providers/Microsoft.Storage/storageAccounts/{name}')
            }
        if self.batch_account.get('key_vault') is not None:
            id = self.normalize_resource_id(
                self.batch_account.pop('key_vault'),
                '/subscriptions/{subscription_id}/resourceGroups/{resource_group}/providers/Microsoft.KeyVault/vaults/{name}')
            url = 'https://' + id.split('/').pop() + '.vault.azure.net/'
            self.batch_account['key_vault_reference'] = {
                'id': id,
                'url': url
            }
        self.batch_account['pool_allocation_mode'] = _snake_to_camel(self.batch_account['pool_allocation_mode'], True)

        response = None

        self.mgmt_client = self.get_mgmt_svc_client(BatchManagementClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        old_response = self.get_batchaccount()

        if not old_response:
            self.log("Batch Account instance doesn't exist")
            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                self.to_do = Actions.Create
        else:
            self.log("Batch Account instance already exists")
            if self.state == 'absent':
                self.to_do = Actions.Delete
            elif self.state == 'present':
                self.results['old'] = old_response
                self.results['new'] = self.batch_account
                if not self.idempotency_check(old_response, self.batch_account):
                    self.to_do = Actions.Update

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.log("Need to Create / Update the Batch Account instance")

            self.results['changed'] = True
            if self.check_mode:
                return self.results

            response = self.create_update_batchaccount()

            self.log("Creation / Update done")
        elif self.to_do == Actions.Delete:
            self.log("Batch Account instance deleted")
            self.results['changed'] = True

            if self.check_mode:
                return self.results

            self.delete_batchaccount()
        else:
            self.log("Batch Account instance unchanged")
            self.results['changed'] = False
            response = old_response

        if self.state == 'present':
            self.results.update({
                'id': response.get('id', None),
                'account_endpoint': response.get('account_endpoint', None)
            })
        return self.results

    def create_update_batchaccount(self):
        '''
        Creates or updates Batch Account with the specified configuration.

        :return: deserialized Batch Account instance state dictionary
        '''
        self.log("Creating / Updating the Batch Account instance {0}".format(self.name))

        try:
            if self.to_do == Actions.Create:
                response = self.mgmt_client.batch_account.create(resource_group_name=self.resource_group,
                                                                 account_name=self.name,
                                                                 parameters=self.batch_account)
            else:
                response = self.mgmt_client.batch_account.update(resource_group_name=self.resource_group,
                                                                 account_name=self.name,
                                                                 tags=self.tags,
                                                                 auto_storage=self.batch_account.get('auto_storage'))
            if isinstance(response, LROPoller) or isinstance(response, AzureOperationPoller):
                response = self.get_poller_result(response)
        except CloudError as exc:
            self.log('Error attempting to create the Batch Account instance.')
            self.fail("Error creating the Batch Account instance: {0}".format(str(exc)))
        return response.as_dict()

    def delete_batchaccount(self):
        '''
        Deletes specified Batch Account instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the Batch Account instance {0}".format(self.name))
        try:
            response = self.mgmt_client.batch_account.delete(resource_group_name=self.resource_group,
                                                             account_name=self.name)
        except CloudError as e:
            self.log('Error attempting to delete the Batch Account instance.')
            self.fail("Error deleting the Batch Account instance: {0}".format(str(e)))

        if isinstance(response, LROPoller) or isinstance(response, AzureOperationPoller):
            response = self.get_poller_result(response)
        return True

    def get_batchaccount(self):
        '''
        Gets the properties of the specified Batch Account
        :return: deserialized Batch Account instance state dictionary
        '''
        self.log("Checking if the Batch Account instance {0} is present".format(self.name))
        found = False
        try:
            response = self.mgmt_client.batch_account.get(resource_group_name=self.resource_group,
                                                          account_name=self.name)
            found = True
            self.log("Response : {0}".format(response))
            self.log("Batch Account instance : {0} found".format(response.name))
        except CloudError as e:
            self.log('Did not find the Batch Account instance.')
        if found is True:
            return response.as_dict()
        return False


def main():
    """Main execution"""
    AzureRMBatchAccount()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2019 Zim Kalinowski, (@zikalino)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}


DOCUMENTATION = '''
---
module: azure_rm_galleryimage
version_added: '2.9'
short_description: Manage Azure SIG Image instance
description:
    - Create, update and delete instance of Azure SIG Image.
options:
    resource_group:
        description:
            - The name of the resource group.
        required: true
        type: str
    gallery_name:
        description:
            - The name of the Shared Image Gallery in which the Image Definition is to be created.
        required: true
        type: str
    name:
        description:
            - The name of the gallery Image Definition to be created or updated.
            - The allowed characters are alphabets and numbers with dots, dashes, and periods allowed in the middle.
            - The maximum length is 80 characters.
        required: true
        type: str
    location:
        description:
            - Resource location.
        type: str
    description:
        description:
            - The description of this gallery Image Definition resource. This property is updatable.
        type: str
    eula:
        description:
            - The Eula agreement for the gallery Image Definition.
        type: str
    privacy_statement_uri:
        description:
            - The privacy statement uri.
        type: str
    release_note_uri:
        description:
            - The release note uri.
        type: str
    os_type:
        description:
            - This property allows you to specify the type of the OS that is included in the disk when creating a VM from a managed image.
        choices:
            - windows
            - linux
        required: true
        type: str
    os_state:
        description:
            - The allowed values for OS State are C(generalized).
        choices:
            - generalized
            - specialized
        required: true
        type: str
    end_of_life_date:
        description:
            - The end of life date of the gallery Image Definition.
            - This property can be used for decommissioning purposes.
            - This property is updatable.
            - Format should be according to ISO-8601, for instance "2019-06-26".
        type: str
    identifier:
        description:
            - Image identifier.
        required: true
        type: dict
        suboptions:
            publisher:
                description:
                    - The name of the gallery Image Definition publisher.
                required: true
                type: str
            offer:
                 description:
                     - The name of the gallery Image Definition offer.
                 required: true
                 type: str
            sku:
                description:
                    - The name of the gallery Image Definition SKU.
                required: true
                type: str
    recommended:
        description:
            - Recommended parameter values.
        type: dict
        suboptions:
            v_cpus:
                description:
                    - Number of virtual CPUs.
                type: dict
                suboptions:
                    min:
                        description:
                            - The minimum number of the resource.
                        type: int
                    max:
                        description:
                            - The maximum number of the resource.
                        type: int
            memory:
                description:
                    - Memory.
                type: dict
                suboptions:
                    min:
                        description:
                            - The minimum number of the resource.
                        type: int
                    max:
                        description:
                            - The maximum number of the resource.
                        type: int
    disallowed:
        description:
            - Disallowed parameter values.
        type: dict
        suboptions:
            disk_types:
                description:
                    - A list of disallowed disk types.
                type: list
    purchase_plan:
        description:
            - Purchase plan.
        type: dict
        suboptions:
            name:
                description:
                    - The plan ID.
                type: str
            publisher:
                description:
                    - The publisher ID.
                type: str
            product:
                description:
                    - The product ID.
                type: str
    state:
        description:
            - Assert the state of the GalleryImage.
            - Use C(present) to create or update an GalleryImage and C(absent) to delete it.
        default: present
        choices:
            - absent
            - present
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
author:
  - Zim Kalinowski (@zikalino)

'''

EXAMPLES = '''
- name: Create or update gallery image
  azure_rm_galleryimage:
    resource_group: myResourceGroup
    gallery_name: myGallery1283
    name: myImage
    location: West US
    os_type: linux
    os_state: generalized
    identifier:
      publisher: myPublisherName
      offer: myOfferName
      sku: mySkuName
'''

RETURN = '''
id:
    description:
        - Resource ID.
    returned: always
    type: str
    sample: "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/myResourceGroup/providers/Microsoft.Compute/galleries/myGalle
           ry1283/images/myImage"
'''

import time
import json
import re
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient
from copy import deepcopy
try:
    from msrestazure.azure_exceptions import CloudError
except ImportError:
    # This is handled in azure_rm_common
    pass


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureRMGalleryImages(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                updatable=False,
                disposition='resourceGroupName',
                required=True
            ),
            gallery_name=dict(
                type='str',
                updatable=False,
                disposition='galleryName',
                required=True
            ),
            name=dict(
                type='str',
                updatable=False,
                disposition='galleryImageName',
                required=True
            ),
            location=dict(
                type='str',
                updatable=False,
                disposition='/'
            ),
            description=dict(
                type='str',
                disposition='/properties/*'
            ),
            eula=dict(
                type='str',
                disposition='/properties/*'
            ),
            privacy_statement_uri=dict(
                type='str',
                disposition='/properties/privacyStatementUri'
            ),
            release_note_uri=dict(
                type='str',
                disposition='/properties/releaseNoteUri'
            ),
            os_type=dict(
                type='str',
                disposition='/properties/osType',
                choices=['windows',
                         'linux']
            ),
            os_state=dict(
                type='str',
                disposition='/properties/osState',
                choices=['generalized',
                         'specialized']
            ),
            end_of_life_date=dict(
                type='str',
                disposition='/properties/endOfLifeDate'
            ),
            identifier=dict(
                type='dict',
                disposition='/properties/*',
                options=dict(
                    publisher=dict(
                        type='str',
                        required=True,
                        updatable=False
                    ),
                    offer=dict(
                        type='str',
                        required=True
                    ),
                    sku=dict(
                        type='str',
                        required=True
                    )
                )
            ),
            recommended=dict(
                type='dict',
                disposition='/properties/*',
                options=dict(
                    v_cpus=dict(
                        type='dict',
                        disposition='vCPUs',
                        options=dict(
                            min=dict(
                                type='int'
                            ),
                            max=dict(
                                type='int'
                            )
                        )
                    ),
                    memory=dict(
                        type='dict',
                        options=dict(
                            min=dict(
                                type='int'
                            ),
                            max=dict(
                                type='int'
                            )
                        )
                    )
                )
            ),
            disallowed=dict(
                type='dict',
                disposition='/properties/*',
                options=dict(
                    disk_types=dict(
                        type='list',
                        disposition='diskTypes'
                    )
                )
            ),
            purchase_plan=dict(
                type='dict',
                disposition='/properties/purchasePlan',
                options=dict(
                    name=dict(
                        type='str'
                    ),
                    publisher=dict(
                        type='str'
                    ),
                    product=dict(
                        type='str'
                    )
                )
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.gallery_name = None
        self.name = None
        self.gallery_image = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.url = None
        self.status_code = [200, 201, 202]
        self.to_do = Actions.NoAction

        self.body = {}
        self.query_parameters = {}
        self.query_parameters['api-version'] = '2019-07-01'
        self.header_parameters = {}
        self.header_parameters['Content-Type'] = 'application/json; charset=utf-8'

        super(AzureRMGalleryImages, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                   supports_check_mode=True,
                                                   supports_tags=True)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.body[key] = kwargs[key]

        self.inflate_parameters(self.module_arg_spec, self.body, 0)

        old_response = None
        response = None

        self.mgmt_client = self.get_mgmt_svc_client(GenericRestClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        resource_group = self.get_resource_group(self.resource_group)

        if 'location' not in self.body:
            self.body['location'] = resource_group.location

        self.url = ('/subscriptions' +
                    '/{{ subscription_id }}' +
                    '/resourceGroups' +
                    '/{{ resource_group }}' +
                    '/providers' +
                    '/Microsoft.Compute' +
                    '/galleries' +
                    '/{{ gallery_name }}' +
                    '/images' +
                    '/{{ image_name }}')
        self.url = self.url.replace('{{ subscription_id }}', self.subscription_id)
        self.url = self.url.replace('{{ resource_group }}', self.resource_group)
        self.url = self.url.replace('{{ gallery_name }}', self.gallery_name)
        self.url = self.url.replace('{{ image_name }}', self.name)

        old_response = self.get_resource()

        if not old_response:
            self.log("GalleryImage instance doesn't exist")

            if self.state == 'absent':
                self.log("Old instance didn't exist")
            else:
                self.to_do = Actions.Create
        else:
            self.log('GalleryImage instance already exists')

            if self.state == 'absent':
                self.to_do = Actions.Delete
            else:
                modifiers = {}
                self.create_compare_modifiers(self.module_arg_spec, '', modifiers)
                self.results['modifiers'] = modifiers
                self.results['compare'] = []
                if not self.default_compare(modifiers, self.body, old_response, '', self.results):
                    self.to_do = Actions.Update

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.log('Need to Create / Update the GalleryImage instance')

            if self.check_mode:
                self.results['changed'] = True
                return self.results

            response = self.create_update_resource()

            # if not old_response:
            self.results['changed'] = True
            # else:
            #     self.results['changed'] = old_response.__ne__(response)
            self.log('Creation / Update done')
        elif self.to_do == Actions.Delete:
            self.log('GalleryImage instance deleted')
            self.results['changed'] = True

            if self.check_mode:
                return self.results

            self.delete_resource()

            # make sure instance is actually deleted, for some Azure resources, instance is hanging around
            # for some time after deletion -- this should be really fixed in Azure
            while self.get_resource():
                time.sleep(20)
        else:
            self.log('GalleryImage instance unchanged')
            self.results['changed'] = False
            response = old_response

        if response:
            self.results["id"] = response["id"]

        return self.results

    def create_update_resource(self):
        # self.log('Creating / Updating the GalleryImage instance {0}'.format(self.))

        try:
            response = self.mgmt_client.query(self.url,
                                              'PUT',
                                              self.query_parameters,
                                              self.header_parameters,
                                              self.body,
                                              self.status_code,
                                              600,
                                              30)
        except CloudError as exc:
            self.log('Error attempting to create the GalleryImage instance.')
            self.fail('Error creating the GalleryImage instance: {0}'.format(str(exc)))

        try:
            response = json.loads(response.text)
        except Exception:
            response = {'text': response.text}

        return response

    def delete_resource(self):
        # self.log('Deleting the GalleryImage instance {0}'.format(self.))
        try:
            response = self.mgmt_client.query(self.url,
                                              'DELETE',
                                              self.query_parameters,
                                              self.header_parameters,
                                              None,
                                              self.status_code,
                                              600,
                                              30)
        except CloudError as e:
            self.log('Error attempting to delete the GalleryImage instance.')
            self.fail('Error deleting the GalleryImage instance: {0}'.format(str(e)))

        return True

    def get_resource(self):
        # self.log('Checking if the GalleryImage instance {0} is present'.format(self.))
        found = False
        try:
            response = self.mgmt_client.query(self.url,
                                              'GET',
                                              self.query_parameters,
                                              self.header_parameters,
                                              None,
                                              self.status_code,
                                              600,
                                              30)
            response = json.loads(response.text)
            found = True
            self.log("Response : {0}".format(response))
            # self.log("AzureFirewall instance : {0} found".format(response.name))
        except CloudError as e:
            self.log('Did not find the AzureFirewall instance.')
        if found is True:
            return response

        return False


def main():
    AzureRMGalleryImages()


if __name__ == '__main__':
    main()



from __future__ import absolute_import, division, print_function

__metaclass__ = type
import socket

from ansible.module_utils.six import iteritems


def search_obj_in_list(name, lst, identifier):
    for o in lst:
        if o[identifier] == name:
            return o
    return None


def flatten_dict(x):
    result = {}
    if not isinstance(x, dict):
        return result

    for key, value in iteritems(x):
        if isinstance(value, dict):
            result.update(flatten_dict(value))
        else:
            result[key] = value

    return result


def validate_ipv4_addr(address):
    address = address.split("/")[0]
    try:
        socket.inet_aton(address)
    except socket.error:
        return False
    return address.count(".") == 3


def validate_ipv6_addr(address):
    address = address.split("/")[0]
    try:
        socket.inet_pton(socket.AF_INET6, address)
    except socket.error:
        return False
    return True


def normalize_interface(name):
    """Return the normalized interface name
    """
    if not name:
        return

    def _get_number(name):
        digits = ""
        for char in name:
            if char.isdigit() or char in "/.":
                digits += char
        return digits

    if name.lower().startswith("et"):
        if_type = "Ethernet"
    elif name.lower().startswith("vl"):
        if_type = "Vlan"
    elif name.lower().startswith("lo"):
        if_type = "loopback"
    elif name.lower().startswith("po"):
        if_type = "port-channel"
    elif name.lower().startswith("nv"):
        if_type = "nve"
    else:
        if_type = None

    number_list = name.split(" ")
    if len(number_list) == 2:
        number = number_list[-1].strip()
    else:
        number = _get_number(name)

    if if_type:
        proper_interface = if_type + number
    else:
        proper_interface = name

    return proper_interface


def get_interface_type(interface):
    """Gets the type of interface
    """
    if interface.upper().startswith("ET"):
        return "ethernet"
    elif interface.upper().startswith("VL"):
        return "svi"
    elif interface.upper().startswith("LO"):
        return "loopback"
    elif interface.upper().startswith("MG"):
        return "management"
    elif interface.upper().startswith("MA"):
        return "management"
    elif interface.upper().startswith("PO"):
        return "portchannel"
    elif interface.upper().startswith("NV"):
        return "nve"
    else:
        return "unknown"


def remove_rsvd_interfaces(interfaces):
    """Exclude reserved interfaces from user management
    """
    if not interfaces:
        return []
    return [
        i for i in interfaces if get_interface_type(i["name"]) != "management"
    ]


def vlan_range_to_list(vlans):
    result = []
    if vlans:
        for part in vlans.split(","):
            if part == "none":
                break
            if "-" in part:
                a, b = part.split("-")
                a, b = int(a), int(b)
                result.extend(range(a, b + 1))
            else:
                a = int(part)
                result.append(a)
        return numerical_sort(result)
    return result


def numerical_sort(string_int_list):
    """Sorts list of integers that are digits in numerical order.
    """

    as_int_list = []

    for vlan in string_int_list:
        as_int_list.append(int(vlan))
    as_int_list.sort()
    return as_int_list



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'certified'}

DOCUMENTATION = r'''
---
module: aci_config_snapshot
short_description: Manage Config Snapshots (config:Snapshot, config:ExportP)
description:
- Manage Config Snapshots on Cisco ACI fabrics.
- Creating new Snapshots is done using the configExportP class.
- Removing Snapshots is done using the configSnapshot class.
options:
  description:
    description:
    - The description for the Config Export Policy.
    type: str
    aliases: [ descr ]
  export_policy:
    description:
    - The name of the Export Policy to use for Config Snapshots.
    type: str
    aliases: [ name ]
  format:
    description:
    - Sets the config backup to be formatted in JSON or XML.
    - The APIC defaults to C(json) when unset.
    type: str
    choices: [ json, xml ]
  include_secure:
    description:
    - Determines if secure information should be included in the backup.
    - The APIC defaults to C(yes) when unset.
    type: bool
  max_count:
    description:
    - Determines how many snapshots can exist for the Export Policy before the APIC starts to rollover.
    - Accepted values range between C(1) and C(10).
    - The APIC defaults to C(3) when unset.
    type: int
  snapshot:
    description:
    - The name of the snapshot to delete.
    type: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci

notes:
- The APIC does not provide a mechanism for naming the snapshots.
- 'Snapshot files use the following naming structure: ce_<config export policy name>-<yyyy>-<mm>-<dd>T<hh>:<mm>:<ss>.<mss>+<hh>:<mm>.'
- 'Snapshot objects use the following naming structure: run-<yyyy>-<mm>-<dd>T<hh>-<mm>-<ss>.'
seealso:
- module: aci_config_rollback
- name: APIC Management Information Model reference
  description: More information about the internal APIC classes B(config:Snapshot) and B(config:ExportP).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Jacob McGill (@jmcgill298)
'''

EXAMPLES = r'''
- name: Create a Snapshot
  cisco.aci.aci_config_snapshot:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: present
    export_policy: config_backup
    max_count: 10
    description: Backups taken before new configs are applied.
  delegate_to: localhost

- name: Query all Snapshots
  cisco.aci.aci_config_snapshot:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Query Snapshots associated with a particular Export Policy
  cisco.aci.aci_config_snapshot:
    host: apic
    username: admin
    password: SomeSecretPassword
    export_policy: config_backup
    state: query
  delegate_to: localhost
  register: query_result

- name: Delete a Snapshot
  cisco.aci.aci_config_snapshot:
    host: apic
    username: admin
    password: SomeSecretPassword
    export_policy: config_backup
    snapshot: run-2017-08-24T17-20-05
    state: absent
  delegate_to: localhost
'''

RETURN = r'''
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
'''

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(
        description=dict(type='str', aliases=['descr']),
        export_policy=dict(type='str', aliases=['name']),  # Not required for querying all objects
        format=dict(type='str', choices=['json', 'xml']),
        include_secure=dict(type='bool'),
        max_count=dict(type='int'),
        snapshot=dict(type='str'),
        state=dict(type='str', choices=['absent', 'present', 'query'], default='present'),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=False,
        required_if=[
            ['state', 'absent', ['export_policy', 'snapshot']],
            ['state', 'present', ['export_policy']],
        ],
    )

    aci = ACIModule(module)

    description = module.params.get('description')
    export_policy = module.params.get('export_policy')
    file_format = module.params.get('format')
    include_secure = aci.boolean(module.params.get('include_secure'))
    max_count = module.params.get('max_count')
    if max_count is not None:
        if max_count in range(1, 11):
            max_count = str(max_count)
        else:
            module.fail_json(msg="Parameter 'max_count' must be a number between 1 and 10")
    snapshot = module.params.get('snapshot')
    if snapshot is not None and not snapshot.startswith('run-'):
        snapshot = 'run-' + snapshot
    state = module.params.get('state')

    if state == 'present':
        aci.construct_url(
            root_class=dict(
                aci_class='configExportP',
                aci_rn='fabric/configexp-{0}'.format(export_policy),
                module_object=export_policy,
                target_filter={'name': export_policy},
            ),
        )

        aci.get_existing()

        aci.payload(
            aci_class='configExportP',
            class_config=dict(
                adminSt='triggered',
                descr=description,
                format=file_format,
                includeSecureFields=include_secure,
                maxSnapshotCount=max_count,
                name=export_policy,
                snapshot='yes',
            ),
        )

        aci.get_diff('configExportP')

        # Create a new Snapshot
        aci.post_config()

    else:
        # Prefix the proper url to export_policy
        if export_policy is not None:
            export_policy = 'uni/fabric/configexp-{0}'.format(export_policy)

        aci.construct_url(
            root_class=dict(
                aci_class='configSnapshotCont',
                aci_rn='backupst/snapshots-[{0}]'.format(export_policy),
                module_object=export_policy,
                target_filter={'name': export_policy},
            ),
            subclass_1=dict(
                aci_class='configSnapshot',
                aci_rn='snapshot-{0}'.format(snapshot),
                module_object=snapshot,
                target_filter={'name': snapshot},
            ),
        )

        aci.get_existing()

        if state == 'absent':
            # Build POST request to used to remove Snapshot
            aci.payload(
                aci_class='configSnapshot',
                class_config=dict(
                    name=snapshot,
                    retire="yes",
                ),
            )

            if aci.existing:
                aci.get_diff('configSnapshot')

                # Mark Snapshot for Deletion
                aci.post_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright: (c) 2018, Johannes Brunswicker <johannes.brunswicker@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import absolute_import, division, print_function

__metaclass__ = type

DOCUMENTATION = '''
---
module: utm_dns_host

author:
    - Johannes Brunswicker (@MatrixCrawler)

short_description: create, update or destroy dns entry in Sophos UTM

description:
    - Create, update or destroy a dns entry in SOPHOS UTM.
    - This module needs to have the REST Ability of the UTM to be activated.


options:
    name:
        description:
          - The name of the object. Will be used to identify the entry
        required: true
    address:
        description:
          - The IPV4 Address of the entry. Can be left empty for automatic resolving.
        default: 0.0.0.0
    address6:
        description:
          - The IPV6 Address of the entry. Can be left empty for automatic resolving.
        default: "::"
    comment:
        description:
          - An optional comment to add to the dns host object
    hostname:
        description:
          - The hostname for the dns host object
    interface:
        description:
          - The reference name of the interface to use. If not provided the default interface will be used
    resolved:
        description:
          - whether the hostname's ipv4 address is already resolved or not
        default: False
        type: bool
    resolved6:
        description:
          - whether the hostname's ipv6 address is already resolved or not
        default: False
        type: bool
    timeout:
        description:
          - the timeout for the utm to resolve the ip address for the hostname again
        default: 0

extends_documentation_fragment:
- community.general.utm

'''

EXAMPLES = """
- name: Create UTM dns host entry
  community.general.utm_dns_host:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestDNSEntry
    hostname: testentry.some.tld
    state: present

- name: Remove UTM dns host entry
  community.general.utm_dns_host:
    utm_host: sophos.host.name
    utm_token: abcdefghijklmno1234
    name: TestDNSEntry
    state: absent
"""

RETURN = """
result:
    description: The utm object that was created
    returned: success
    type: complex
    contains:
        _ref:
            description: The reference name of the object
            type: str
        _locked:
            description: Whether or not the object is currently locked
            type: bool
        name:
            description: The name of the object
            type: str
        address:
            description: The ipv4 address of the object
            type: str
        address6:
            description: The ipv6 address of the object
            type: str
        comment:
            description: The comment string
            type: str
        hostname:
            description: The hostname of the object
            type: str
        interface:
            description: The reference name of the interface the object is associated with
            type: str
        resolved:
            description: Whether the ipv4 address is resolved or not
            type: bool
        resolved6:
            description: Whether the ipv6 address is resolved or not
            type: bool
        timeout:
            description: The timeout until a new resolving will be attempted
            type: int
"""

from ansible_collections.community.general.plugins.module_utils.utm_utils import UTM, UTMModule
from ansible.module_utils._text import to_native


def main():
    endpoint = "network/dns_host"
    key_to_check_for_changes = ["comment", "hostname", "interface"]
    module = UTMModule(
        argument_spec=dict(
            name=dict(type='str', required=True),
            address=dict(type='str', required=False, default='0.0.0.0'),
            address6=dict(type='str', required=False, default='::'),
            comment=dict(type='str', required=False, default=""),
            hostname=dict(type='str', required=False),
            interface=dict(type='str', required=False, default=""),
            resolved=dict(type='bool', required=False, default=False),
            resolved6=dict(type='bool', required=False, default=False),
            timeout=dict(type='int', required=False, default=0),
        )
    )
    try:
        UTM(module, endpoint, key_to_check_for_changes).execute()
    except Exception as e:
        module.fail_json(msg=to_native(e))


if __name__ == '__main__':
    main()



# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#

from ansible_collections.community.network.tests.unit.compat import unittest
from ansible_collections.community.network.tests.unit.compat.mock import patch, Mock
import sys
sys.modules['cvprac'] = Mock()
sys.modules['cvprac.cvp_client'] = Mock()
sys.modules['cvprac.cvp_client_errors'] = Mock()
from ansible_collections.community.network.plugins.modules.network.cloudvision import cv_server_provision


class MockException(Exception):
    pass


class TestCvServerProvision(unittest.TestCase):
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.CvpApiError',
           new_callable=lambda: MockException)
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.server_configurable_configlet')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.switch_in_compliance')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.switch_info')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.connect')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.AnsibleModule')
    def test_main_module_args(self, mock_module, mock_connect, mock_info,
                              mock_comp, mock_server_conf, mock_exception):
        ''' Test main module args.
        '''
        mock_module_object = Mock()
        mock_module_object.params = dict(action='show', switch_name='eos')
        mock_module_object.fail_json.side_effect = SystemExit('Exiting')
        mock_module.return_value = mock_module_object
        mock_connect.return_value = 'Client'
        mock_info.side_effect = mock_exception('Error Getting Info')
        argument_spec = dict(
            host=dict(required=True),
            port=dict(required=False, default=None),
            protocol=dict(default='https', choices=['http', 'https']),
            username=dict(required=True),
            password=dict(required=True, no_log=True),
            server_name=dict(required=True),
            switch_name=dict(required=True),
            switch_port=dict(required=True),
            port_vlan=dict(required=False, default=None),
            template=dict(require=True),
            action=dict(default='show', choices=['show', 'add', 'remove']),
            auto_run=dict(type='bool', default=False),
        )
        self.assertRaises(SystemExit, cv_server_provision.main)
        mock_module.assert_called_with(argument_spec=argument_spec,
                                       supports_check_mode=False)
        self.assertEqual(mock_connect.call_count, 1)
        self.assertEqual(mock_info.call_count, 1)
        mock_comp.assert_not_called()
        mock_server_conf.assert_not_called()
        mock_module_object.fail_json.assert_called_with(msg='Error Getting Info')

    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.CvpApiError',
           new_callable=lambda: MockException)
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.server_configurable_configlet')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.switch_in_compliance')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.switch_info')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.connect')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.AnsibleModule')
    def test_main_no_switch_configlet(self, mock_module, mock_connect,
                                      mock_info, mock_comp, mock_server_conf,
                                      mock_exception):
        ''' Test main fails if switch has no configlet for Ansible to edit.
        '''
        mock_module_object = Mock()
        mock_module_object.params = dict(action='add', switch_name='eos')
        mock_module_object.fail_json.side_effect = SystemExit('Exiting')
        mock_module.return_value = mock_module_object
        mock_connect.return_value = 'Client'
        mock_info.return_value = 'Info'
        mock_server_conf.return_value = None
        self.assertRaises(SystemExit, cv_server_provision.main)
        self.assertEqual(mock_connect.call_count, 1)
        self.assertEqual(mock_info.call_count, 1)
        self.assertEqual(mock_comp.call_count, 1)
        self.assertEqual(mock_server_conf.call_count, 1)
        mock_module_object.fail_json.assert_called_with(
            msg='Switch eos has no configurable server ports.')

    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.CvpApiError',
           new_callable=lambda: MockException)
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.port_configurable')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.server_configurable_configlet')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.switch_in_compliance')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.switch_info')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.connect')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.AnsibleModule')
    def test_main_port_not_in_config(self, mock_module, mock_connect, mock_info,
                                     mock_comp, mock_server_conf,
                                     mock_port_conf, mock_exception):
        ''' Test main fails if user specified port not in configlet.
        '''
        mock_module_object = Mock()
        mock_module_object.params = dict(action='add', switch_name='eos',
                                         switch_port='3')
        mock_module_object.fail_json.side_effect = SystemExit('Exiting')
        mock_module.return_value = mock_module_object
        mock_connect.return_value = 'Client'
        mock_info.return_value = 'Info'
        mock_server_conf.return_value = 'Configlet'
        mock_port_conf.return_value = None
        self.assertRaises(SystemExit, cv_server_provision.main)
        self.assertEqual(mock_connect.call_count, 1)
        self.assertEqual(mock_info.call_count, 1)
        self.assertEqual(mock_comp.call_count, 1)
        self.assertEqual(mock_server_conf.call_count, 1)
        self.assertEqual(mock_port_conf.call_count, 1)
        mock_module_object.fail_json.assert_called_with(
            msg='Port 3 is not configurable as a server port on switch eos.')

    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.configlet_action')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.port_configurable')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.server_configurable_configlet')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.switch_in_compliance')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.switch_info')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.connect')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.AnsibleModule')
    def test_main_show(self, mock_module, mock_connect, mock_info, mock_comp,
                       mock_server_conf, mock_port_conf, mock_conf_action):
        ''' Test main good with show action.
        '''
        mock_module_object = Mock()
        mock_module_object.params = dict(action='show', switch_name='eos',
                                         switch_port='3', auto_run=False)
        mock_module.return_value = mock_module_object
        mock_connect.return_value = 'Client'
        mock_info.return_value = 'Info'
        mock_server_conf.return_value = 'Configlet'
        mock_port_conf.return_value = 'Port'
        mock_conf_action.return_value = dict()
        cv_server_provision.main()
        self.assertEqual(mock_connect.call_count, 1)
        self.assertEqual(mock_info.call_count, 1)
        mock_comp.assert_not_called()
        self.assertEqual(mock_server_conf.call_count, 1)
        self.assertEqual(mock_port_conf.call_count, 1)
        self.assertEqual(mock_conf_action.call_count, 1)
        mock_module_object.fail_json.assert_not_called()
        return_dict = dict(changed=False, switchInfo='Info',
                           switchConfigurable=True, portConfigurable=True,
                           taskCreated=False, taskExecuted=False,
                           taskCompleted=False)
        mock_module_object.exit_json.assert_called_with(**return_dict)

    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.configlet_action')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.port_configurable')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.server_configurable_configlet')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.switch_in_compliance')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.switch_info')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.connect')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.AnsibleModule')
    def test_main_add_no_auto_run(self, mock_module, mock_connect, mock_info,
                                  mock_comp, mock_server_conf, mock_port_conf,
                                  mock_conf_action):
        ''' Test main good with add action and no auto_run.
        '''
        mock_module_object = Mock()
        mock_module_object.params = dict(action='add', switch_name='eos',
                                         switch_port='3', auto_run=False)
        mock_module.return_value = mock_module_object
        mock_connect.return_value = 'Client'
        mock_info.return_value = 'Info'
        mock_server_conf.return_value = 'Configlet'
        mock_port_conf.return_value = 'Port'
        mock_conf_action.return_value = dict(taskCreated=True)
        cv_server_provision.main()
        self.assertEqual(mock_connect.call_count, 1)
        self.assertEqual(mock_info.call_count, 1)
        self.assertEqual(mock_comp.call_count, 1)
        self.assertEqual(mock_server_conf.call_count, 1)
        self.assertEqual(mock_port_conf.call_count, 1)
        self.assertEqual(mock_conf_action.call_count, 1)
        mock_module_object.fail_json.assert_not_called()
        return_dict = dict(changed=False, switchInfo='Info',
                           switchConfigurable=True, portConfigurable=True,
                           taskCreated=True, taskExecuted=False,
                           taskCompleted=False)
        mock_module_object.exit_json.assert_called_with(**return_dict)

    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.wait_for_task_completion')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.configlet_update_task')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.configlet_action')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.port_configurable')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.server_configurable_configlet')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.switch_in_compliance')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.switch_info')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.connect')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.AnsibleModule')
    def test_main_add_auto_run(self, mock_module, mock_connect, mock_info,
                               mock_comp, mock_server_conf, mock_port_conf,
                               mock_conf_action, mock_conf_task, mock_wait):
        ''' Test main good with add and auto_run. Config updated, task created.
        '''
        mock_module_object = Mock()
        mock_module_object.params = dict(action='add', switch_name='eos',
                                         switch_port='3', auto_run=True)
        mock_module.return_value = mock_module_object
        mock_client_object = Mock()
        mock_connect.return_value = mock_client_object
        mock_info.return_value = 'Info'
        mock_server_conf.return_value = 'Configlet'
        mock_port_conf.return_value = 'Port'
        mock_conf_action.return_value = dict(taskCreated=True, changed=True)
        mock_conf_task.return_value = '7'
        mock_wait.return_value = True
        cv_server_provision.main()
        self.assertEqual(mock_connect.call_count, 1)
        self.assertEqual(mock_info.call_count, 1)
        self.assertEqual(mock_comp.call_count, 1)
        self.assertEqual(mock_server_conf.call_count, 1)
        self.assertEqual(mock_port_conf.call_count, 1)
        self.assertEqual(mock_conf_action.call_count, 1)
        self.assertEqual(mock_conf_task.call_count, 1)
        self.assertEqual(mock_wait.call_count, 1)
        mock_module_object.fail_json.assert_not_called()
        return_dict = dict(changed=True, switchInfo='Info', taskId='7',
                           switchConfigurable=True, portConfigurable=True,
                           taskCreated=True, taskExecuted=True,
                           taskCompleted=True)
        mock_module_object.exit_json.assert_called_with(**return_dict)

    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.wait_for_task_completion')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.configlet_update_task')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.configlet_action')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.port_configurable')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.server_configurable_configlet')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.switch_in_compliance')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.switch_info')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.connect')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.AnsibleModule')
    def test_main_add_auto_run_no_task(self, mock_module, mock_connect,
                                       mock_info, mock_comp, mock_server_conf,
                                       mock_port_conf, mock_conf_action, mock_conf_task,
                                       mock_wait):
        ''' Test main good with add and auto_run. Config not updated, no task.
        '''
        mock_module_object = Mock()
        mock_module_object.params = dict(action='add', switch_name='eos',
                                         switch_port='3', auto_run=True)
        mock_module.return_value = mock_module_object
        mock_client_object = Mock()
        mock_connect.return_value = mock_client_object
        mock_info.return_value = 'Info'
        mock_server_conf.return_value = 'Configlet'
        mock_port_conf.return_value = 'Port'
        mock_conf_action.return_value = dict(taskCreated=True, changed=False)
        mock_conf_task.return_value = None
        cv_server_provision.main()
        self.assertEqual(mock_connect.call_count, 1)
        self.assertEqual(mock_info.call_count, 1)
        self.assertEqual(mock_comp.call_count, 1)
        self.assertEqual(mock_server_conf.call_count, 1)
        self.assertEqual(mock_port_conf.call_count, 1)
        self.assertEqual(mock_conf_action.call_count, 1)
        self.assertEqual(mock_conf_task.call_count, 1)
        mock_wait.assert_not_called()
        mock_module_object.fail_json.assert_not_called()
        return_dict = dict(changed=False, switchInfo='Info',
                           switchConfigurable=True, portConfigurable=True,
                           taskCreated=False, taskExecuted=False,
                           taskCompleted=False)
        mock_module_object.exit_json.assert_called_with(**return_dict)

    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.CvpClient')
    def test_connect_good(self, mock_client):
        ''' Test connect success.
        '''
        module = Mock()
        module.params = dict(host='host', username='username',
                             password='password', protocol='https', port='10')
        connect_mock = Mock()
        mock_client.return_value = connect_mock
        client = cv_server_provision.connect(module)
        self.assertIsInstance(client, Mock)
        self.assertEqual(mock_client.call_count, 1)
        connect_mock.connect.assert_called_once_with(['host'], 'username',
                                                     'password', port='10',
                                                     protocol='https')
        module.fail_json.assert_not_called()

    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.CvpLoginError',
           new_callable=lambda: MockException)
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.CvpClient')
    def test_connect_fail(self, mock_client, mock_exception):
        ''' Test connect failure with login error.
        '''
        module = Mock()
        module.params = dict(host='host', username='username',
                             password='password', protocol='https', port='10')
        module.fail_json.side_effect = SystemExit
        connect_mock = Mock()
        connect_mock.connect.side_effect = mock_exception('Login Error')
        mock_client.return_value = connect_mock
        self.assertRaises(SystemExit, cv_server_provision.connect, module)
        self.assertEqual(connect_mock.connect.call_count, 1)
        module.fail_json.assert_called_once_with(msg='Login Error')

    def test_switch_info_good(self):
        ''' Test switch_info success.
        '''
        module = Mock()
        module.params = dict(switch_name='eos')
        module.client.api.get_device_by_name.return_value = dict(fqdn='eos')
        info = cv_server_provision.switch_info(module)
        self.assertEqual(module.client.api.get_device_by_name.call_count, 1)
        self.assertEqual(info['fqdn'], 'eos')
        module.fail_json.assert_not_called()

    def test_switch_info_no_switch(self):
        ''' Test switch_info fails.
        '''
        module = Mock()
        module.params = dict(switch_name='eos')
        module.client.api.get_device_by_name.return_value = None
        info = cv_server_provision.switch_info(module)
        self.assertEqual(module.client.api.get_device_by_name.call_count, 1)
        self.assertEqual(info, None)
        module.fail_json.assert_called_once_with(
            msg="Device with name 'eos' does not exist.")

    def test_switch_in_compliance_good(self):
        ''' Test switch_in_compliance good.
        '''
        module = Mock()
        module.client.api.check_compliance.return_value = dict(
            complianceCode='0000')
        sw_info = dict(key='key', type='type', fqdn='eos')
        cv_server_provision.switch_in_compliance(module, sw_info)
        self.assertEqual(module.client.api.check_compliance.call_count, 1)
        module.fail_json.assert_not_called()

    def test_switch_in_compliance_fail(self):
        ''' Test switch_in_compliance fail.
        '''
        module = Mock()
        module.client.api.check_compliance.return_value = dict(
            complianceCode='0001')
        sw_info = dict(key='key', type='type', fqdn='eos')
        cv_server_provision.switch_in_compliance(module, sw_info)
        self.assertEqual(module.client.api.check_compliance.call_count, 1)
        module.fail_json.assert_called_with(
            msg='Switch eos is not in compliance.'
                ' Returned compliance code 0001.')

    def test_server_configurable_configlet_good(self):
        ''' Test server_configurable_configlet good.
        '''
        module = Mock()
        module.params = dict(switch_name='eos')
        configlets = [dict(name='configlet1', info='line'),
                      dict(name='eos-server', info='info')]
        module.client.api.get_configlets_by_device_id.return_value = configlets
        sw_info = dict(key='key', type='type', fqdn='eos')
        result = cv_server_provision.server_configurable_configlet(module,
                                                                   sw_info)
        self.assertEqual(module.client.api.get_configlets_by_device_id.call_count, 1)
        self.assertIsNotNone(result)
        self.assertEqual(result['name'], 'eos-server')
        self.assertEqual(result['info'], 'info')

    def test_server_configurable_configlet_not_configurable(self):
        ''' Test server_configurable_configlet fail. No server configlet.
        '''
        module = Mock()
        module.params = dict(switch_name='eos')
        configlets = [dict(name='configlet1', info='line'),
                      dict(name='configlet2', info='info')]
        module.client.api.get_configlets_by_device_id.return_value = configlets
        sw_info = dict(key='key', type='type', fqdn='eos')
        result = cv_server_provision.server_configurable_configlet(module, sw_info)
        self.assertEqual(module.client.api.get_configlets_by_device_id.call_count, 1)
        self.assertIsNone(result)

    def test_server_configurable_configlet_no_configlets(self):
        ''' Test server_configurable_configlet fail. No switch configlets.
        '''
        module = Mock()
        module.params = dict(switch_name='eos')
        module.client.api.get_configlets_by_device_id.return_value = []
        sw_info = dict(key='key', type='type', fqdn='eos')
        result = cv_server_provision.server_configurable_configlet(module,
                                                                   sw_info)
        self.assertEqual(module.client.api.get_configlets_by_device_id.call_count, 1)
        self.assertIsNone(result)

    def test_port_configurable_good(self):
        ''' Test port_configurable user provided switch port in configlet.
        '''
        module = Mock()
        module.params = dict(switch_name='eos', switch_port='3')
        config = '!\ninterface Ethernet3\n!\ninterface Ethernet4\n!'
        configlet = dict(name='eos-server', config=config)
        result = cv_server_provision.port_configurable(module, configlet)
        self.assertTrue(result)

    def test_port_configurable_fail(self):
        ''' Test port_configurable user provided switch port not in configlet.
        '''
        module = Mock()
        module.params = dict(switch_name='eos', switch_port='2')
        config = '!\ninterface Ethernet3\n!\ninterface Ethernet4\n!'
        configlet = dict(name='eos-server', config=config)
        result = cv_server_provision.port_configurable(module, configlet)
        self.assertFalse(result)

    def test_port_configurable_fail_no_config(self):
        ''' Test port_configurable configlet empty.
        '''
        module = Mock()
        module.params = dict(switch_name='eos', switch_port='2')
        config = ''
        configlet = dict(name='eos-server', config=config)
        result = cv_server_provision.port_configurable(module, configlet)
        self.assertFalse(result)

    def test_configlet_action_show_blank_config(self):
        ''' Test configlet_action show returns current port configuration.
        '''
        module = Mock()
        module.params = dict(action='show', switch_name='eos', switch_port='3')
        config = '!\ninterface Ethernet3\n!\ninterface Ethernet4\n!'
        configlet = dict(name='eos-server', key='key', config=config)
        result = cv_server_provision.configlet_action(module, configlet)
        self.assertIsNotNone(result)
        self.assertEqual(result['currentConfigBlock'], 'interface Ethernet3\n!')
        module.client.api.update_configlet.assert_not_called()

    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.config_from_template')
    def test_configlet_action_add_with_task(self, mock_template):
        ''' Test configlet_action add with change updates configlet and adds
            proper info to return data. Including task spawned info.
        '''
        module = Mock()
        module.params = dict(action='add', switch_name='eos', switch_port='3')
        config = '!\ninterface Ethernet3\n!\ninterface Ethernet4\n!'
        configlet = dict(name='eos-server', key='key', config=config)
        template_config = ('interface Ethernet3\n   description Host eos'
                           ' managed by Ansible and Jinja template\n'
                           '   load-interval 30\n'
                           '   switchport\n'
                           '   switchport mode trunk\n'
                           '   no shutdown\n!')
        mock_template.return_value = template_config
        update_return = dict(data='Configlet eos-server successfully updated'
                                  ' and task initiated.')
        module.client.api.update_configlet.return_value = update_return
        result = cv_server_provision.configlet_action(module, configlet)
        self.assertIsNotNone(result)
        self.assertEqual(result['oldConfigBlock'], 'interface Ethernet3\n!')
        full_config = '!\n' + template_config + '\ninterface Ethernet4\n!'
        self.assertEqual(result['fullConfig'], full_config)
        self.assertEqual(result['updateConfigletResponse'],
                         update_return['data'])
        self.assertTrue(result['changed'])
        self.assertTrue(result['taskCreated'])
        self.assertEqual(module.client.api.update_configlet.call_count, 1)

    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.config_from_template')
    def test_configlet_action_add_no_task(self, mock_template):
        ''' Test configlet_action add that doesn't change configlet adds proper
            info to return data. Does not including any task info.
        '''
        module = Mock()
        module.params = dict(action='add', switch_name='eos', switch_port='3')
        config = ('!\ninterface Ethernet3\n   description test\n'
                  '!\ninterface Ethernet4\n!')
        configlet = dict(name='eos-server', key='key', config=config)
        template_config = 'interface Ethernet3\n   description test\n!'
        mock_template.return_value = template_config
        update_return = dict(data='Configlet eos-server successfully updated.')
        module.client.api.update_configlet.return_value = update_return
        result = cv_server_provision.configlet_action(module, configlet)
        self.assertIsNotNone(result)
        self.assertEqual(result['oldConfigBlock'],
                         'interface Ethernet3\n   description test\n!')
        self.assertEqual(result['fullConfig'], config)
        self.assertEqual(result['updateConfigletResponse'],
                         update_return['data'])
        self.assertNotIn('changed', result)
        self.assertNotIn('taskCreated', result)
        self.assertEqual(module.client.api.update_configlet.call_count, 1)

    def test_configlet_action_remove_with_task(self):
        ''' Test configlet_action remove with change updates configlet and adds
            proper info to return data. Including task spawned info.
        '''
        module = Mock()
        module.params = dict(action='remove', switch_name='eos',
                             switch_port='3')
        config = ('!\ninterface Ethernet3\n   description test\n'
                  '!\ninterface Ethernet4\n!')
        configlet = dict(name='eos-server', key='key', config=config)
        update_return = dict(data='Configlet eos-server successfully updated'
                                  ' and task initiated.')
        module.client.api.update_configlet.return_value = update_return
        result = cv_server_provision.configlet_action(module, configlet)
        self.assertIsNotNone(result)
        self.assertEqual(result['oldConfigBlock'],
                         'interface Ethernet3\n   description test\n!')
        full_config = '!\ninterface Ethernet3\n!\ninterface Ethernet4\n!'
        self.assertEqual(result['fullConfig'], full_config)
        self.assertEqual(result['updateConfigletResponse'],
                         update_return['data'])
        self.assertTrue(result['changed'])
        self.assertTrue(result['taskCreated'])
        self.assertEqual(module.client.api.update_configlet.call_count, 1)

    def test_configlet_action_remove_no_task(self):
        ''' Test configlet_action with remove that doesn't change configlet and
            adds proper info to return data. Does not including any task info.
        '''
        module = Mock()
        module.params = dict(action='remove', switch_name='eos',
                             switch_port='3')
        config = '!\ninterface Ethernet3\n!\ninterface Ethernet4\n!'
        configlet = dict(name='eos-server', key='key', config=config)
        update_return = dict(data='Configlet eos-server successfully updated.')
        module.client.api.update_configlet.return_value = update_return
        result = cv_server_provision.configlet_action(module, configlet)
        self.assertIsNotNone(result)
        self.assertEqual(result['oldConfigBlock'], 'interface Ethernet3\n!')
        self.assertEqual(result['fullConfig'], config)
        self.assertEqual(result['updateConfigletResponse'],
                         update_return['data'])
        self.assertNotIn('changed', result)
        self.assertNotIn('taskCreated', result)
        self.assertEqual(module.client.api.update_configlet.call_count, 1)

    def test_current_config_empty_config(self):
        ''' Test current_config with empty config for port
        '''
        module = Mock()
        module.params = dict(switch_name='eos', switch_port='4')
        config = '!\ninterface Ethernet3\n!\ninterface Ethernet4'
        result = cv_server_provision.current_config(module, config)
        self.assertIsNotNone(result)
        self.assertEqual(result, 'interface Ethernet4')

    def test_current_config_with_config(self):
        ''' Test current_config with config for port
        '''
        module = Mock()
        module.params = dict(switch_name='eos', switch_port='3')
        config = ('!\ninterface Ethernet3\n   description test\n'
                  '!\ninterface Ethernet4\n!')
        result = cv_server_provision.current_config(module, config)
        self.assertIsNotNone(result)
        self.assertEqual(result, 'interface Ethernet3\n   description test\n!')

    def test_current_config_no_match(self):
        ''' Test current_config with no entry for port
        '''
        module = Mock()
        module.fail_json.side_effect = SystemExit
        module.params = dict(switch_name='eos', switch_port='2')
        config = '!\ninterface Ethernet3\n   description test\n!'
        self.assertRaises(SystemExit, cv_server_provision.current_config,
                          module, config)

    def test_valid_template_true(self):
        ''' Test valid_template true
        '''
        template = 'interface Ethernet3\n   description test\n!'
        result = cv_server_provision.valid_template('3', template)
        self.assertTrue(result)

    def test_valid_template_false(self):
        ''' Test valid_template false
        '''
        template = 'interface Ethernet3\n   description test\n!'
        result = cv_server_provision.valid_template('4', template)
        self.assertFalse(result)

    @patch('jinja2.DebugUndefined')
    @patch('jinja2.Environment')
    @patch('jinja2.FileSystemLoader')
    def test_config_from_template_no_template(self, mock_file_sys, mock_env,
                                              mock_debug):
        ''' Test config_from_template good. No template.
        '''
        module = Mock()
        module.fail_json.side_effect = SystemExit
        module.params = dict(switch_name='eos', switch_port='3',
                             server_name='new', template='jinja.j2')
        mock_file_sys.return_value = 'file'
        mock_debug.return_value = 'debug'
        env_mock = Mock()
        env_mock.get_template.return_value = None
        mock_env.return_value = env_mock
        self.assertRaises(SystemExit, cv_server_provision.config_from_template,
                          module)
        self.assertEqual(mock_file_sys.call_count, 1)
        self.assertEqual(mock_env.call_count, 1)
        self.assertEqual(module.fail_json.call_count, 1)

    @patch('jinja2.meta.find_undeclared_variables')
    @patch('jinja2.DebugUndefined')
    @patch('jinja2.Environment')
    @patch('jinja2.FileSystemLoader')
    def test_config_from_template_good_no_vlan(self, mock_file_sys, mock_env, mock_debug,
                                               mock_find):
        ''' Test config_from_template good. No port_vlan.
        '''
        module = Mock()
        module.params = dict(switch_name='eos', switch_port='3',
                             server_name='new', template='jinja.j2')
        mock_file_sys.return_value = 'file'
        mock_debug.return_value = 'debug'
        template_mock = Mock()
        template_mock.render.return_value = ('interface Ethernet3\n'
                                             '   description test\n'
                                             '   switchport\n'
                                             '   switchport mode trunk\n'
                                             '   no shutdown\n!')
        env_mock = Mock()
        env_mock.loader.get_source.return_value = ['one', 'two']
        env_mock.parse.return_value = 'parsed'
        env_mock.get_template.return_value = template_mock
        mock_env.return_value = env_mock
        mock_find.return_value = dict(server_name=None, switch_port=None)
        result = cv_server_provision.config_from_template(module)
        self.assertIsNotNone(result)
        expected = ('interface Ethernet3\n'
                    '   description test\n'
                    '   switchport\n'
                    '   switchport mode trunk\n'
                    '   no shutdown\n!')
        self.assertEqual(result, expected)
        self.assertEqual(mock_file_sys.call_count, 1)
        self.assertEqual(mock_env.call_count, 1)
        module.fail_json.assert_not_called()

    @patch('jinja2.meta.find_undeclared_variables')
    @patch('jinja2.DebugUndefined')
    @patch('jinja2.Environment')
    @patch('jinja2.FileSystemLoader')
    def test_config_from_template_good_vlan(self, mock_file_sys, mock_env, mock_debug,
                                            mock_find):
        ''' Test config_from_template good. With port_vlan.
        '''
        module = Mock()
        module.params = dict(switch_name='eos', switch_port='3',
                             server_name='new', template='jinja.j2', port_vlan='7')
        mock_file_sys.return_value = 'file'
        mock_debug.return_value = 'debug'
        template_mock = Mock()
        template_mock.render.return_value = ('interface Ethernet3\n'
                                             '   description test\n'
                                             '   switchport\n'
                                             '   switchport access vlan 7\n'
                                             '   no shutdown\n!')
        env_mock = Mock()
        env_mock.loader.get_source.return_value = ['one', 'two']
        env_mock.parse.return_value = 'parsed'
        env_mock.get_template.return_value = template_mock
        mock_env.return_value = env_mock
        mock_find.return_value = dict(server_name=None, switch_port=None,
                                      port_vlan=None)
        result = cv_server_provision.config_from_template(module)
        self.assertIsNotNone(result)
        expected = ('interface Ethernet3\n'
                    '   description test\n'
                    '   switchport\n'
                    '   switchport access vlan 7\n'
                    '   no shutdown\n!')
        self.assertEqual(result, expected)
        self.assertEqual(mock_file_sys.call_count, 1)
        self.assertEqual(mock_env.call_count, 1)
        module.fail_json.assert_not_called()

    @patch('jinja2.meta.find_undeclared_variables')
    @patch('jinja2.DebugUndefined')
    @patch('jinja2.Environment')
    @patch('jinja2.FileSystemLoader')
    def test_config_from_template_fail_wrong_port(self, mock_file_sys, mock_env,
                                                  mock_debug, mock_find):
        ''' Test config_from_template fail. Wrong port number in template.
        '''
        module = Mock()
        module.params = dict(switch_name='eos', switch_port='4',
                             server_name='new', template='jinja.j2')
        mock_file_sys.return_value = 'file'
        mock_debug.return_value = 'debug'
        template_mock = Mock()
        template_mock.render.return_value = ('interface Ethernet3\n'
                                             '   description test\n!')
        env_mock = Mock()
        env_mock.loader.get_source.return_value = ['one', 'two']
        env_mock.parse.return_value = 'parsed'
        env_mock.get_template.return_value = template_mock
        mock_env.return_value = env_mock
        mock_find.return_value = dict(server_name=None, switch_port=None)
        result = cv_server_provision.config_from_template(module)
        self.assertIsNotNone(result)
        expected = 'interface Ethernet3\n   description test\n!'
        self.assertEqual(result, expected)
        self.assertEqual(mock_file_sys.call_count, 1)
        self.assertEqual(mock_env.call_count, 1)
        module.fail_json.assert_called_with(msg='Template content does not'
                                                ' configure proper interface'
                                                ' - %s' % expected)

    @patch('jinja2.meta.find_undeclared_variables')
    @patch('jinja2.DebugUndefined')
    @patch('jinja2.Environment')
    @patch('jinja2.FileSystemLoader')
    def test_config_from_template_fail_no_vlan(self, mock_file_sys, mock_env,
                                               mock_debug, mock_find):
        ''' Test config_from_template fail. Template needs vlan but none provided.
        '''
        module = Mock()
        module.params = dict(switch_name='eos', switch_port='3',
                             server_name='new', template='jinja.j2',
                             port_vlan=None)
        mock_file_sys.return_value = 'file'
        mock_debug.return_value = 'debug'
        template_mock = Mock()
        template_mock.render.return_value = ('interface Ethernet3\n'
                                             '   description test\n!')
        env_mock = Mock()
        env_mock.loader.get_source.return_value = ['one', 'two']
        env_mock.parse.return_value = 'parsed'
        env_mock.get_template.return_value = template_mock
        mock_env.return_value = env_mock
        mock_find.return_value = dict(server_name=None, switch_port=None,
                                      port_vlan=None)
        result = cv_server_provision.config_from_template(module)
        self.assertIsNotNone(result)
        expected = 'interface Ethernet3\n   description test\n!'
        self.assertEqual(result, expected)
        self.assertEqual(mock_file_sys.call_count, 1)
        self.assertEqual(mock_env.call_count, 1)
        module.fail_json.assert_called_with(msg='Template jinja.j2 requires a'
                                                ' vlan. Please re-run with vlan'
                                                ' number provided.')

    def test_updated_configlet_content_add(self):
        ''' Test updated_configlet_content. Add config.
        '''
        module = Mock()
        module.params = dict(switch_name='eos', switch_port='3')
        existing_config = '!\ninterface Ethernet3\n!\ninterface Ethernet4\n!'
        new_config_block = 'interface Ethernet3\n   description test\n!'
        result = cv_server_provision.updated_configlet_content(module,
                                                               existing_config,
                                                               new_config_block)
        expected = ('!\ninterface Ethernet3\n   description test\n'
                    '!\ninterface Ethernet4\n!')
        self.assertEqual(result, expected)
        module.fail_json.assert_not_called()

    def test_updated_configlet_content_remove(self):
        ''' Test updated_configlet_content. Remove config.
        '''
        module = Mock()
        module.params = dict(switch_name='eos', switch_port='3')
        existing_config = ('!\ninterface Ethernet3\n   description test\n'
                           '!\ninterface Ethernet4')
        new_config_block = 'interface Ethernet3\n!'
        result = cv_server_provision.updated_configlet_content(module,
                                                               existing_config,
                                                               new_config_block)
        expected = '!\ninterface Ethernet3\n!\ninterface Ethernet4'
        self.assertEqual(result, expected)
        module.fail_json.assert_not_called()

    def test_updated_configlet_content_no_match(self):
        ''' Test updated_configlet_content. Interface not in config.
        '''
        module = Mock()
        module.fail_json.side_effect = SystemExit
        module.params = dict(switch_name='eos', switch_port='2')
        existing_config = '!\ninterface Ethernet3\n   description test\n!'
        new_config_block = 'interface Ethernet3\n!'
        self.assertRaises(SystemExit,
                          cv_server_provision.updated_configlet_content,
                          module, existing_config, new_config_block)

    @patch('time.sleep')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.switch_info')
    def test_configlet_update_task_good_one_try(self, mock_info, mock_sleep):
        ''' Test configlet_update_task gets task after one try.
        '''
        module = Mock()
        task = dict(data=dict(WORKFLOW_ACTION='Configlet Push'),
                    description='Configlet Assign',
                    workOrderId='7')
        device_info = dict(taskIdList=[task])
        mock_info.return_value = device_info
        result = cv_server_provision.configlet_update_task(module)
        self.assertEqual(result, '7')
        mock_sleep.assert_not_called()
        self.assertEqual(mock_info.call_count, 1)

    @patch('time.sleep')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.switch_info')
    def test_configlet_update_task_good_three_tries(self, mock_info, mock_sleep):
        ''' Test configlet_update_task gets task on third try.
        '''
        module = Mock()
        task1 = dict(data=dict(WORKFLOW_ACTION='Configlet Push'),
                     description='Configlet Assign',
                     workOrderId='7')
        task2 = dict(data=dict(WORKFLOW_ACTION='Nonsense'),
                     description='Configlet Assign',
                     workOrderId='700')
        device_info = dict(taskIdList=[task1, task2])
        mock_info.side_effect = [dict(), dict(), device_info]
        result = cv_server_provision.configlet_update_task(module)
        self.assertEqual(result, '7')
        self.assertEqual(mock_sleep.call_count, 2)
        self.assertEqual(mock_info.call_count, 3)

    @patch('time.sleep')
    @patch('ansible_collections.community.network.plugins.modules.network.cloudvision.cv_server_provision.switch_info')
    def test_configlet_update_task_no_task(self, mock_info, mock_sleep):
        ''' Test configlet_update_task does not get task after three tries.
        '''
        module = Mock()
        mock_info.side_effect = [dict(), dict(), dict()]
        result = cv_server_provision.configlet_update_task(module)
        self.assertIsNone(result)
        self.assertEqual(mock_sleep.call_count, 3)
        self.assertEqual(mock_info.call_count, 3)

    @patch('time.sleep')
    def test_wait_for_task_completion_good_one_try(self, mock_time):
        ''' Test wait_for_task_completion completed. One Try.
        '''
        module = Mock()
        module.client.api.get_task_by_id.return_value = dict(
            workOrderUserDefinedStatus='Completed')
        result = cv_server_provision.wait_for_task_completion(module, '7')
        self.assertTrue(result)
        self.assertEqual(module.client.api.get_task_by_id.call_count, 1)
        module.fail_json.assert_not_called()
        mock_time.assert_not_called()

    @patch('time.sleep')
    def test_wait_for_task_completion_good_three_tries(self, mock_time):
        ''' Test wait_for_task_completion completed. Three tries.
        '''
        module = Mock()
        try_one_two = dict(workOrderUserDefinedStatus='Pending')
        try_three = dict(workOrderUserDefinedStatus='Completed')
        module.client.api.get_task_by_id.side_effect = [try_one_two,
                                                        try_one_two, try_three]
        result = cv_server_provision.wait_for_task_completion(module, '7')
        self.assertTrue(result)
        self.assertEqual(module.client.api.get_task_by_id.call_count, 3)
        module.fail_json.assert_not_called()
        self.assertEqual(mock_time.call_count, 2)

    @patch('time.sleep')
    def test_wait_for_task_completion_fail(self, mock_time):
        ''' Test wait_for_task_completion failed.
        '''
        module = Mock()
        try_one = dict(workOrderUserDefinedStatus='Failed')
        try_two = dict(workOrderUserDefinedStatus='Completed')
        module.client.api.get_task_by_id.side_effect = [try_one, try_two]
        result = cv_server_provision.wait_for_task_completion(module, '7')
        self.assertTrue(result)
        self.assertEqual(module.client.api.get_task_by_id.call_count, 2)
        text = ('Task 7 has reported status Failed. Please consult the CVP'
                ' admins for more information.')
        module.fail_json.assert_called_with(msg=text)
        self.assertEqual(mock_time.call_count, 1)
